@@ -19,12 +19,20 @@ email                : brush.tyler@gmail.com
  *                                                                         *
  ***************************************************************************/
 """
+from builtins import str
+from builtins import range
 
-from qgis.PyQt.QtCore import Qt, QObject, QSettings, pyqtSignal
+from qgis.PyQt.QtCore import Qt, QObject, pyqtSignal
 from qgis.PyQt.QtWidgets import QApplication, QAction, QMenu, QInputDialog, QMessageBox
 from qgis.PyQt.QtGui import QKeySequence, QIcon
 
 from qgis.gui import QgsMessageBar
+from qgis.core import (
+    Qgis,
+    QgsApplication,
+    QgsSettings,
+    QgsWkbTypes
+)
 from ..db_plugins import createDbPlugin
 
 
@@ -38,8 +46,8 @@ class BaseError(Exception):
         else:
             msg = e
 
-        if not isinstance(msg, unicode):
-            msg = unicode(msg, 'utf-8', 'replace')  # convert from utf8 and replace errors (if any)
+        if not isinstance(msg, str):
+            msg = str(msg, 'utf-8', 'replace')  # convert from utf8 and replace errors (if any)
 
         self.msg = msg
         Exception.__init__(self, msg)
@@ -47,9 +55,6 @@ class BaseError(Exception):
     def __unicode__(self):
         return self.msg
 
-    def __str__(self):
-        return unicode(self).encode('utf-8')
-
 
 class InvalidDataException(BaseError):
     pass
@@ -63,15 +68,15 @@ class DbError(BaseError):
 
     def __init__(self, e, query=None):
         BaseError.__init__(self, e)
-        self.query = unicode(query) if query is not None else None
+        self.query = str(query) if query is not None else None
 
     def __unicode__(self):
         if self.query is None:
             return BaseError.__unicode__(self)
 
-        msg = QApplication.translate("DBManagerPlugin", "Error:\n%s") % BaseError.__unicode__(self)
+        msg = QApplication.translate("DBManagerPlugin", "Error:\n{0}").format(BaseError.__unicode__(self))
         if self.query:
-            msg += QApplication.translate("DBManagerPlugin", "\n\nQuery:\n%s") % self.query
+            msg += QApplication.translate("DBManagerPlugin", "\n\nQuery:\n{0}").format(self.query)
         return msg
 
 
@@ -88,6 +93,9 @@ class DBPlugin(QObject):
     def __del__(self):
         pass  # print "DBPlugin.__del__", self.connName
 
+    def connectionIcon(self):
+        return QgsApplication.getThemeIcon("/mIconDbSchema.svg")
+
     def connectionName(self):
         return self.connName
 
@@ -100,7 +108,7 @@ class DBPlugin(QObject):
         return DatabaseInfo(None)
 
     def connect(self, parent=None):
-        raise NotImplemented
+        raise NotImplementedError('Needs to be implemented by subclasses')
 
     def connectToUri(self, uri):
         self.db = self.databasesFactory(self, uri)
@@ -117,7 +125,7 @@ class DBPlugin(QObject):
         return self.connect(self.parent())
 
     def remove(self):
-        settings = QSettings()
+        settings = QgsSettings()
         settings.beginGroup(u"/%s/%s" % (self.connectionSettingsKey(), self.connectionName()))
         settings.remove("")
         self.deleted.emit()
@@ -125,7 +133,7 @@ class DBPlugin(QObject):
 
     @classmethod
     def addConnection(self, conn_name, uri):
-        raise NotImplemented
+        raise NotImplementedError('Needs to be implemented by subclasses')
 
     @classmethod
     def icon(self):
@@ -155,7 +163,7 @@ class DBPlugin(QObject):
     def connections(self):
         # get the list of connections
         conn_list = []
-        settings = QSettings()
+        settings = QgsSettings()
         settings.beginGroup(self.connectionSettingsKey())
         for name in settings.childGroups():
             conn_list.append(createDbPlugin(self.typeName(), name))
@@ -167,14 +175,14 @@ class DBPlugin(QObject):
 
     @classmethod
     def addConnectionActionSlot(self, item, action, parent):
-        raise NotImplemented
+        raise NotImplementedError('Needs to be implemented by subclasses')
 
     def removeActionSlot(self, item, action, parent):
         QApplication.restoreOverrideCursor()
         try:
-            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "hey!"),
+            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "DB Manager"),
                                        QApplication.translate("DBManagerPlugin",
-                                                              "Really remove connection to %s?") % item.connectionName(),
+                                                              "Really remove connection to {0}?").format(item.connectionName()),
                                        QMessageBox.Yes | QMessageBox.No)
             if res != QMessageBox.Yes:
                 return
@@ -253,6 +261,11 @@ class Database(DbItemObject):
 
         return SqlResultModel(self, sql, parent)
 
+    def sqlResultModelAsync(self, sql, parent):
+        from .data_model import SqlResultModelAsync
+
+        return SqlResultModelAsync(self, sql, parent)
+
     def columnUniqueValuesModel(self, col, table, limit=10):
         l = ""
         if limit is not None:
@@ -274,7 +287,7 @@ class Database(DbItemObject):
                     q = 1
                     while "_subq_%d_" % q in sql:
                         q += 1
-                    sql = "SELECT %s AS _uid_,* FROM (%s) AS _subq_%d_" % (uniqueFct, sql, q)
+                    sql = u"SELECT %s AS _uid_,* FROM (%s\n) AS _subq_%d_" % (uniqueFct, sql, q)
                     uniqueCol = "_uid_"
 
         uri = self.uri()
@@ -293,7 +306,7 @@ class Database(DbItemObject):
     def registerSubPluginActions(self, mainWindow):
         # load plugins!
         try:
-            exec (u"from .%s.plugins import load" % self.dbplugin().typeName(), globals())
+            exec(u"from .%s.plugins import load" % self.dbplugin().typeName(), globals())
         except ImportError:
             pass
         else:
@@ -305,34 +318,34 @@ class Database(DbItemObject):
                                   self.reconnectActionSlot)
 
         if self.schemas() is not None:
-            action = QAction(QApplication.translate("DBManagerPlugin", "&Create schema"), self)
+            action = QAction(QApplication.translate("DBManagerPlugin", "&Create Schema…"), self)
             mainWindow.registerAction(action, QApplication.translate("DBManagerPlugin", "&Schema"),
                                       self.createSchemaActionSlot)
-            action = QAction(QApplication.translate("DBManagerPlugin", "&Delete (empty) schema"), self)
+            action = QAction(QApplication.translate("DBManagerPlugin", "&Delete (Empty) Schema"), self)
             mainWindow.registerAction(action, QApplication.translate("DBManagerPlugin", "&Schema"),
                                       self.deleteSchemaActionSlot)
 
-        action = QAction(QApplication.translate("DBManagerPlugin", "Delete selected item"), self)
+        action = QAction(QApplication.translate("DBManagerPlugin", "Delete Selected Item"), self)
         mainWindow.registerAction(action, None, self.deleteActionSlot)
         action.setShortcuts(QKeySequence.Delete)
 
-        action = QAction(QIcon(":/db_manager/actions/create_table"),
-                         QApplication.translate("DBManagerPlugin", "&Create table"), self)
+        action = QAction(QgsApplication.getThemeIcon("/mActionCreateTable.svg"),
+                         QApplication.translate("DBManagerPlugin", "&Create Table…"), self)
         mainWindow.registerAction(action, QApplication.translate("DBManagerPlugin", "&Table"),
                                   self.createTableActionSlot)
-        action = QAction(QIcon(":/db_manager/actions/edit_table"),
-                         QApplication.translate("DBManagerPlugin", "&Edit table"), self)
+        action = QAction(QgsApplication.getThemeIcon("/mActionEditTable.svg"),
+                         QApplication.translate("DBManagerPlugin", "&Edit Table…"), self)
         mainWindow.registerAction(action, QApplication.translate("DBManagerPlugin", "&Table"), self.editTableActionSlot)
-        action = QAction(QIcon(":/db_manager/actions/del_table"),
-                         QApplication.translate("DBManagerPlugin", "&Delete table/view"), self)
+        action = QAction(QgsApplication.getThemeIcon("/mActionDeleteTable.svg"),
+                         QApplication.translate("DBManagerPlugin", "&Delete Table/View…"), self)
         mainWindow.registerAction(action, QApplication.translate("DBManagerPlugin", "&Table"),
                                   self.deleteTableActionSlot)
-        action = QAction(QApplication.translate("DBManagerPlugin", "&Empty table"), self)
+        action = QAction(QApplication.translate("DBManagerPlugin", "&Empty Table…"), self)
         mainWindow.registerAction(action, QApplication.translate("DBManagerPlugin", "&Table"),
                                   self.emptyTableActionSlot)
 
         if self.schemas() is not None:
-            action = QAction(QApplication.translate("DBManagerPlugin", "&Move to schema"), self)
+            action = QAction(QApplication.translate("DBManagerPlugin", "&Move to Schema"), self)
             action.setMenu(QMenu(mainWindow))
 
             def invoke_callback():
@@ -354,7 +367,7 @@ class Database(DbItemObject):
         else:
             QApplication.restoreOverrideCursor()
             parent.infoBar.pushMessage(QApplication.translate("DBManagerPlugin", "Cannot delete the selected item."),
-                                       QgsMessageBar.INFO, parent.iface.messageTimeout())
+                                       Qgis.Info, parent.iface.messageTimeout())
             QApplication.setOverrideCursor(Qt.WaitCursor)
 
     def createSchemaActionSlot(self, item, action, parent):
@@ -363,7 +376,7 @@ class Database(DbItemObject):
             if not isinstance(item, (DBPlugin, Schema, Table)) or item.database() is None:
                 parent.infoBar.pushMessage(
                     QApplication.translate("DBManagerPlugin", "No database selected or you are not connected to it."),
-                    QgsMessageBar.INFO, parent.iface.messageTimeout())
+                    Qgis.Info, parent.iface.messageTimeout())
                 return
             (schema, ok) = QInputDialog.getText(parent, QApplication.translate("DBManagerPlugin", "New schema"),
                                                 QApplication.translate("DBManagerPlugin", "Enter new schema name"))
@@ -380,11 +393,11 @@ class Database(DbItemObject):
             if not isinstance(item, Schema):
                 parent.infoBar.pushMessage(
                     QApplication.translate("DBManagerPlugin", "Select an empty schema for deletion."),
-                    QgsMessageBar.INFO, parent.iface.messageTimeout())
+                    Qgis.Info, parent.iface.messageTimeout())
                 return
-            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "hey!"),
+            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "DB Manager"),
                                        QApplication.translate("DBManagerPlugin",
-                                                              "Really delete schema %s?") % item.name,
+                                                              "Really delete schema {0}?").format(item.name),
                                        QMessageBox.Yes | QMessageBox.No)
             if res != QMessageBox.Yes:
                 return
@@ -411,7 +424,7 @@ class Database(DbItemObject):
         if not hasattr(item, 'database') or item.database() is None:
             parent.infoBar.pushMessage(
                 QApplication.translate("DBManagerPlugin", "No database selected or you are not connected to it."),
-                QgsMessageBar.INFO, parent.iface.messageTimeout())
+                Qgis.Info, parent.iface.messageTimeout())
             return
         from ..dlg_create_table import DlgCreateTable
 
@@ -422,8 +435,8 @@ class Database(DbItemObject):
         QApplication.restoreOverrideCursor()
         try:
             if not isinstance(item, Table) or item.isView:
-                parent.infoBar.pushMessage(QApplication.translate("DBManagerPlugin", "Select a table for editation."),
-                                           QgsMessageBar.INFO, parent.iface.messageTimeout())
+                parent.infoBar.pushMessage(QApplication.translate("DBManagerPlugin", "Select a table to edit."),
+                                           Qgis.Info, parent.iface.messageTimeout())
                 return
             from ..dlg_table_properties import DlgTableProperties
 
@@ -437,11 +450,11 @@ class Database(DbItemObject):
             if not isinstance(item, Table):
                 parent.infoBar.pushMessage(
                     QApplication.translate("DBManagerPlugin", "Select a table/view for deletion."),
-                    QgsMessageBar.INFO, parent.iface.messageTimeout())
+                    Qgis.Info, parent.iface.messageTimeout())
                 return
-            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "hey!"),
+            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "DB Manager"),
                                        QApplication.translate("DBManagerPlugin",
-                                                              "Really delete table/view %s?") % item.name,
+                                                              "Really delete table/view {0}?").format(item.name),
                                        QMessageBox.Yes | QMessageBox.No)
             if res != QMessageBox.Yes:
                 return
@@ -455,11 +468,11 @@ class Database(DbItemObject):
         try:
             if not isinstance(item, Table) or item.isView:
                 parent.infoBar.pushMessage(QApplication.translate("DBManagerPlugin", "Select a table to empty it."),
-                                           QgsMessageBar.INFO, parent.iface.messageTimeout())
+                                           Qgis.Info, parent.iface.messageTimeout())
                 return
-            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "hey!"),
+            res = QMessageBox.question(parent, QApplication.translate("DBManagerPlugin", "DB Manager"),
                                        QApplication.translate("DBManagerPlugin",
-                                                              "Really delete all items from table %s?") % item.name,
+                                                              "Really delete all items from table {0}?").format(item.name),
                                        QMessageBox.Yes | QMessageBox.No)
             if res != QMessageBox.Yes:
                 return
@@ -482,7 +495,7 @@ class Database(DbItemObject):
         try:
             if not isinstance(item, Table):
                 parent.infoBar.pushMessage(QApplication.translate("DBManagerPlugin", "Select a table/view."),
-                                           QgsMessageBar.INFO, parent.iface.messageTimeout())
+                                           Qgis.Info, parent.iface.messageTimeout())
                 return
         finally:
             QApplication.setOverrideCursor(Qt.WaitCursor)
@@ -509,8 +522,28 @@ class Database(DbItemObject):
     def tables(self, schema=None, sys_tables=False):
         tables = self.connector.getTables(schema.name if schema else None, sys_tables)
         if tables is not None:
-            tables = [self.tablesFactory(x, self, schema) for x in tables]
-        return tables
+            ret = []
+            for t in tables:
+                table = self.tablesFactory(t, self, schema)
+                ret.append(table)
+
+                # Similarly to what to browser does, if the geom type is generic geometry,
+                # we additionnly add three copies of the layer to allow importing
+                if isinstance(table, VectorTable):
+                    if table.geomType == 'GEOMETRY':
+                        point_table = self.tablesFactory(t, self, schema)
+                        point_table.geomType = 'POINT'
+                        ret.append(point_table)
+
+                        line_table = self.tablesFactory(t, self, schema)
+                        line_table.geomType = 'LINESTRING'
+                        ret.append(line_table)
+
+                        poly_table = self.tablesFactory(t, self, schema)
+                        poly_table.geomType = 'POLYGON'
+                        ret.append(poly_table)
+
+        return ret
 
     def createTable(self, table, fields, schema=None):
         field_defs = [x.definition() for x in fields]
@@ -596,7 +629,7 @@ class Schema(DbItemObject):
 
 
 class Table(DbItemObject):
-    TableType, VectorType, RasterType = range(3)
+    TableType, VectorType, RasterType = list(range(3))
 
     def __init__(self, db, schema=None, parent=None):
         DbItemObject.__init__(self, db)
@@ -673,8 +706,15 @@ class Table(DbItemObject):
         uri = self.database().uri()
         schema = self.schemaName() if self.schemaName() else ''
         geomCol = self.geomColumn if self.type in [Table.VectorType, Table.RasterType] else ""
-        uniqueCol = self.getValidQGisUniqueFields(True) if self.isView else None
+        uniqueCol = self.getValidQgisUniqueFields(True) if self.isView else None
         uri.setDataSource(schema, self.name, geomCol if geomCol else None, None, uniqueCol.name if uniqueCol else "")
+        uri.setSrid(str(self.srid))
+        for f in self.fields():
+            if f.primaryKey:
+                uri.setKeyColumn(f.name)
+                break
+        uri.setWkbType(QgsWkbTypes.parseType(self.geomType))
+
         return uri
 
     def mimeUri(self):
@@ -690,9 +730,9 @@ class Table(DbItemObject):
             return QgsRasterLayer(uri, self.name, provider)
         return QgsVectorLayer(uri, self.name, provider)
 
-    def getValidQGisUniqueFields(self, onlyOne=False):
-        """ list of fields valid to load the table as layer in QGis canvas.
-                QGis automatically search for a valid unique field, so it's
+    def getValidQgisUniqueFields(self, onlyOne=False):
+        """ list of fields valid to load the table as layer in Qgis canvas.
+                Qgis automatically search for a valid unique field, so it's
                 needed only for queries and views """
 
         ret = []
@@ -723,8 +763,8 @@ class Table(DbItemObject):
     def tableDataModel(self, parent):
         pass
 
-    def tableFieldsFactory(self):
-        return None
+    def tableFieldsFactory(self, row, table):
+        raise NotImplementedError('Needs to be implemented by subclasses')
 
     def fields(self):
         if self._fields is None:
@@ -873,7 +913,7 @@ class Table(DbItemObject):
             self.refresh()
 
     def runAction(self, action):
-        action = unicode(action)
+        action = str(action)
 
         if action.startswith("rows/"):
             if action == "rows/count":
@@ -884,7 +924,7 @@ class Table(DbItemObject):
             parts = action.split('/')
             trigger_action = parts[1]
 
-            msg = QApplication.translate("DBManagerPlugin", "Do you want to %s all triggers?") % trigger_action
+            msg = QApplication.translate("DBManagerPlugin", "Do you want to {0} all triggers?").format(trigger_action)
             QApplication.restoreOverrideCursor()
             try:
                 if QMessageBox.question(None, QApplication.translate("DBManagerPlugin", "Table triggers"), msg,
@@ -905,7 +945,7 @@ class Table(DbItemObject):
             trigger_name = parts[1]
             trigger_action = parts[2]
 
-            msg = QApplication.translate("DBManagerPlugin", "Do you want to %s trigger %s?") % (
+            msg = QApplication.translate("DBManagerPlugin", "Do you want to {0} trigger {1}?").format(
                 trigger_action, trigger_name)
             QApplication.restoreOverrideCursor()
             try:
@@ -995,13 +1035,13 @@ class VectorTable(Table):
             self.refresh()
 
     def runAction(self, action):
-        action = unicode(action)
+        action = str(action)
 
         if action.startswith("spatialindex/"):
             parts = action.split('/')
             spatialIndex_action = parts[1]
 
-            msg = QApplication.translate("DBManagerPlugin", "Do you want to %s spatial index for field %s?") % (
+            msg = QApplication.translate("DBManagerPlugin", "Do you want to {0} spatial index for field {1}?").format(
                 spatialIndex_action, self.geomColumn)
             QApplication.restoreOverrideCursor()
             try:
@@ -1116,7 +1156,7 @@ class TableConstraint(TableSubItemObject):
 
     """ class that represents a constraint of a table (relation) """
 
-    TypeCheck, TypeForeignKey, TypePrimaryKey, TypeUnique, TypeExclusion, TypeUnknown = range(6)
+    TypeCheck, TypeForeignKey, TypePrimaryKey, TypeUnique, TypeExclusion, TypeUnknown = list(range(6))
     types = {"c": TypeCheck, "f": TypeForeignKey, "p": TypePrimaryKey, "u": TypeUnique, "x": TypeExclusion}
 
     onAction = {"a": "NO ACTION", "r": "RESTRICT", "c": "CASCADE", "n": "SET NULL", "d": "SET DEFAULT"}
