@@ -23,7 +23,7 @@ __copyright__ = '(C) 2014, Radoslaw Guzinski'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 import os
 
@@ -60,9 +60,6 @@ class buildvrt(GdalAlgorithm):
     RESAMPLING = 'RESAMPLING'
     SRC_NODATA = 'SRC_NODATA'
 
-    RESOLUTION_OPTIONS = ['average', 'highest', 'lowest']
-    RESAMPLING_OPTIONS = ['nearest', 'bilinear', 'cubic', 'cubicspline', 'lanczos', 'average', 'mode']
-
     def __init__(self):
         super().__init__()
 
@@ -83,12 +80,24 @@ class buildvrt(GdalAlgorithm):
             def defaultFileExtension(self):
                 return 'vrt'
 
+        self.RESAMPLING_OPTIONS = ((self.tr('Nearest Neighbour'), 'nearest'),
+                                   (self.tr('Bilinear'), 'bilinear'),
+                                   (self.tr('Cubic Convolution'), 'cubic'),
+                                   (self.tr('B-Spline Convolution'), 'cubicspline'),
+                                   (self.tr('Lanczos Windowed Sinc'), 'lanczos'),
+                                   (self.tr('Average'), 'average'),
+                                   (self.tr('Mode'), 'mode'))
+
+        self.RESOLUTION_OPTIONS = ((self.tr('Average'), 'average'),
+                                   (self.tr('Highest'), 'highest'),
+                                   (self.tr('Lowest'), 'lowest'))
+
         self.addParameter(QgsProcessingParameterMultipleLayers(self.INPUT,
                                                                self.tr('Input layers'),
                                                                QgsProcessing.TypeRaster))
         self.addParameter(QgsProcessingParameterEnum(self.RESOLUTION,
                                                      self.tr('Resolution'),
-                                                     options=self.RESOLUTION_OPTIONS,
+                                                     options=[i[0] for i in self.RESOLUTION_OPTIONS],
                                                      defaultValue=0))
         self.addParameter(QgsProcessingParameterBoolean(self.SEPARATE,
                                                         self.tr('Place each input file into a separate band'),
@@ -111,7 +120,7 @@ class buildvrt(GdalAlgorithm):
 
         resampling = QgsProcessingParameterEnum(self.RESAMPLING,
                                                 self.tr('Resampling algorithm'),
-                                                options=self.RESAMPLING_OPTIONS,
+                                                options=[i[0] for i in self.RESAMPLING_OPTIONS],
                                                 defaultValue=0)
         resampling.setFlags(resampling.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
         self.addParameter(resampling)
@@ -146,7 +155,7 @@ class buildvrt(GdalAlgorithm):
     def getConsoleCommands(self, parameters, context, feedback, executing=True):
         arguments = []
         arguments.append('-resolution')
-        arguments.append(self.RESOLUTION_OPTIONS[self.parameterAsEnum(parameters, self.RESOLUTION, context)])
+        arguments.append(self.RESOLUTION_OPTIONS[self.parameterAsEnum(parameters, self.RESOLUTION, context)][1])
         if self.parameterAsBool(parameters, buildvrt.SEPARATE, context):
             arguments.append('-separate')
         if self.parameterAsBool(parameters, buildvrt.PROJ_DIFFERENCE, context):
@@ -158,7 +167,7 @@ class buildvrt(GdalAlgorithm):
             arguments.append('-a_srs')
             arguments.append(GdalUtils.gdal_crs_string(crs))
         arguments.append('-r')
-        arguments.append(self.RESAMPLING_OPTIONS[self.parameterAsEnum(parameters, self.RESAMPLING, context)])
+        arguments.append(self.RESAMPLING_OPTIONS[self.parameterAsEnum(parameters, self.RESAMPLING, context)][1])
 
         if self.SRC_NODATA in parameters and parameters[self.SRC_NODATA] not in (None, ''):
             nodata = self.parameterAsString(parameters, self.SRC_NODATA, context)
