@@ -23,7 +23,7 @@ __copyright__ = '(C) 2010, Michael Minn'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 from qgis.PyQt.QtCore import QVariant
 from qgis.core import (QgsExpression,
@@ -48,7 +48,7 @@ class SelectByAttribute(QgisAlgorithm):
     OUTPUT = 'OUTPUT'
 
     OPERATORS = ['=',
-                 '≠',
+                 '<>',
                  '>',
                  '>=',
                  '<',
@@ -82,9 +82,9 @@ class SelectByAttribute(QgisAlgorithm):
         self.operators = ['=',
                           '≠',
                           '>',
-                          '>=',
+                          '≥',
                           '<',
-                          '<=',
+                          '≤',
                           self.tr('begins with'),
                           self.tr('contains'),
                           self.tr('is null'),
@@ -104,14 +104,14 @@ class SelectByAttribute(QgisAlgorithm):
                                                       self.tr('Selection attribute'),
                                                       parentLayerParameterName=self.INPUT))
         self.addParameter(QgsProcessingParameterEnum(self.OPERATOR,
-                                                     self.tr('Operator'), self.operators))
+                                                     self.tr('Operator'), self.operators, defaultValue=0))
         self.addParameter(QgsProcessingParameterString(self.VALUE,
                                                        self.tr('Value'),
                                                        optional=True))
         self.addParameter(QgsProcessingParameterEnum(self.METHOD,
                                                      self.tr('Modify current selection by'),
                                                      self.methods,
-                                                     0))
+                                                     defaultValue=0))
 
         self.addOutput(QgsProcessingOutputVectorLayer(self.OUTPUT, self.tr('Selected (attribute)')))
 
