@@ -22,7 +22,7 @@ __copyright__ = '(C) 2014, Michael Douchin'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 from qgis.core import (QgsExpression,
                        QgsProcessing,
@@ -66,7 +66,7 @@ class SelectByExpression(QgisAlgorithm):
         self.addParameter(QgsProcessingParameterExpression(self.EXPRESSION,
                                                            self.tr('Expression'), parentLayerParameterName=self.INPUT))
         self.addParameter(QgsProcessingParameterEnum(self.METHOD,
-                                                     self.tr('Modify current selection by'), self.methods, 0))
+                                                     self.tr('Modify current selection by'), self.methods, defaultValue=0))
 
         self.addOutput(QgsProcessingOutputVectorLayer(self.OUTPUT, self.tr('Selected (attribute)')))
 
