@@ -24,7 +24,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 import os
 import shutil
@@ -180,7 +180,7 @@ class SagaAlgorithm(SagaAlgorithmBase):
                 if isinstance(parameters[param.name()], str):
                     if parameters[param.name()].lower().endswith('sdat'):
                         self.exportedLayers[param.name()] = parameters[param.name()][:-4] + 'sgrd'
-                    if parameters[param.name()].lower().endswith('sgrd'):
+                    elif parameters[param.name()].lower().endswith('sgrd'):
                         self.exportedLayers[param.name()] = parameters[param.name()]
                     else:
                         layer = self.parameterAsRasterLayer(parameters, param.name(), context)
@@ -225,7 +225,7 @@ class SagaAlgorithm(SagaAlgorithmBase):
                     for i, layer in enumerate(layers):
                         if layer.source().lower().endswith('sdat'):
                             files.append(layer.source()[:-4] + 'sgrd')
-                        if layer.source().lower().endswith('sgrd'):
+                        elif layer.source().lower().endswith('sgrd'):
                             files.append(layer.source())
                         else:
                             exportCommand = self.exportRasterLayer(param.name(), layer)
