@@ -19,13 +19,14 @@
 #include <QWidget>
 
 #include "qgsmaptooledit.h"
-#include "qgsvectorlayer.h"
-
+#include "qgsvertexmarker.h"
+#include "qgis_app.h"
+#include "qgsgeometry.h"
+#include "qgsfeatureid.h"
 
 class QgsDoubleSpinBox;
 class QHBoxLayout;
 class QgsSpinBox;
-class QgsVertexMarker;
 
 class APP_EXPORT QgsAngleMagnetWidget : public QWidget
 {
@@ -33,19 +34,19 @@ class APP_EXPORT QgsAngleMagnetWidget : public QWidget
 
   public:
 
-    explicit QgsAngleMagnetWidget( const QString& label = QString(), QWidget *parent = nullptr );
-
-    ~QgsAngleMagnetWidget();
+    explicit QgsAngleMagnetWidget( const QString &label = QString(), QWidget *parent = nullptr );
 
     void setAngle( double angle );
-
-    double angle();
-
+    double angle() const;
     void setMagnet( int magnet );
+    int magnet() const;
+
+    QgsDoubleSpinBox *editor() const {return mAngleSpinBox;}
 
   signals:
     void angleChanged( double angle );
     void angleEditingFinished( double angle );
+    void angleEditingCanceled();
 
 
   public slots:
@@ -57,61 +58,65 @@ class APP_EXPORT QgsAngleMagnetWidget : public QWidget
     void angleSpinBoxValueChanged( double angle );
 
   private:
-    QHBoxLayout* mLayout;
-    QgsDoubleSpinBox* mAngleSpinBox;
-    QgsSpinBox* mMagnetSpinBox;
+    QHBoxLayout *mLayout = nullptr;
+    QgsDoubleSpinBox *mAngleSpinBox = nullptr;
+    QgsSpinBox *mMagnetSpinBox = nullptr;
 };
 
 
-/** Map tool to rotate features */
+//! Map tool to rotate features
 class APP_EXPORT QgsMapToolRotateFeature: public QgsMapToolEdit
 {
     Q_OBJECT
   public:
-    QgsMapToolRotateFeature( QgsMapCanvas* canvas );
-    virtual ~QgsMapToolRotateFeature();
+    QgsMapToolRotateFeature( QgsMapCanvas *canvas );
+    ~QgsMapToolRotateFeature() override;
 
-    virtual void canvasMoveEvent( QgsMapMouseEvent* e ) override;
+    void canvasMoveEvent( QgsMapMouseEvent *e ) override;
 
-    virtual void canvasReleaseEvent( QgsMapMouseEvent* e ) override;
+    void canvasReleaseEvent( QgsMapMouseEvent *e ) override;
 
     //! called when map tool is being deactivated
     void deactivate() override;
 
     void activate() override;
 
+    //! catch escape when active to cancel selection
+    void keyReleaseEvent( QKeyEvent *e ) override;
+
   private slots:
     void updateRubberband( double rotation );
 
     void applyRotation( double rotation );
+    void cancel();
 
   private:
 
-    QgsGeometry rotateGeometry( QgsGeometry geom, QgsPoint point, double angle );
-    QgsPoint rotatePoint( QgsPoint point, double angle );
+    QgsGeometry rotateGeometry( QgsGeometry geom, QgsPointXY point, double angle );
+    QgsPointXY rotatePoint( QgsPointXY point, double angle );
     void deleteRubberband();
     void createRotationWidget();
     void deleteRotationWidget();
 
-    /** Start point of the move in map coordinates*/
-    QgsPoint mStartPointMapCoords;
+    //! Start point of the move in map coordinates
+    QgsPointXY mStartPointMapCoords;
     QPointF mInitialPos;
 
-    /** Rubberband that shows the feature being moved*/
-    QgsRubberBand* mRubberBand;
+    //! Rubberband that shows the feature being moved
+    QgsRubberBand *mRubberBand = nullptr;
 
-    /** Id of moved feature*/
+    //! Id of moved feature
     QgsFeatureIds mRotatedFeatures;
     double mRotation;
     double mRotationOffset;
 
     QPoint mStPoint;
-    QgsVertexMarker* mAnchorPoint;
+    std::unique_ptr<QgsVertexMarker> mAnchorPoint = nullptr;
 
     bool mRotationActive;
 
-    /** Shows current angle value and allows numerical editing*/
-    QgsAngleMagnetWidget* mRotationWidget;
+    //! Shows current angle value and allows numerical editing
+    QgsAngleMagnetWidget *mRotationWidget = nullptr;
 };
 
 #endif
