@@ -16,54 +16,50 @@
 #include "qgsdistancearea.h"
 #include "qgslogger.h"
 #include "qgsmapcanvas.h"
-#include "qgsmaprenderer.h"
 #include "qgsmaptopixel.h"
 #include "qgsrubberband.h"
 #include "qgsvectorlayer.h"
 #include "qgssnappingutils.h"
 #include "qgstolerance.h"
-
+#include "qgsexception.h"
 #include "qgsmeasuredialog.h"
 #include "qgsmeasuretool.h"
-#include "qgscursors.h"
+#include "qgsmessagelog.h"
+#include "qgssettings.h"
+#include "qgsproject.h"
+#include "qgssnapindicator.h"
+#include "qgsmapmouseevent.h"
 
 #include <QMessageBox>
-#include <QMouseEvent>
-#include <QSettings>
 
-QgsMeasureTool::QgsMeasureTool( QgsMapCanvas* canvas, bool measureArea )
-    : QgsMapTool( canvas )
-    , mWrongProjectProjection( false )
+
+QgsMeasureTool::QgsMeasureTool( QgsMapCanvas *canvas, bool measureArea )
+  : QgsMapTool( canvas )
+  , mMeasureArea( measureArea )
+  , mSnapIndicator( new QgsSnapIndicator( canvas ) )
 {
-  mMeasureArea = measureArea;
+  mRubberBand = new QgsRubberBand( canvas, mMeasureArea ? QgsWkbTypes::PolygonGeometry : QgsWkbTypes::LineGeometry );
+  mRubberBandPoints = new QgsRubberBand( canvas, QgsWkbTypes::PointGeometry );
 
-  mRubberBand = new QgsRubberBand( canvas, mMeasureArea ? QGis::Polygon : QGis::Line );
-  mRubberBandPoints = new QgsRubberBand( canvas, QGis::Point );
-
-  QPixmap myCrossHairQPixmap = QPixmap(( const char ** ) cross_hair_cursor );
-  mCursor = QCursor( myCrossHairQPixmap, 8, 8 );
-
-  mDone = true;
   // Append point we will move
-  mPoints.append( QgsPoint( 0, 0 ) );
+  mPoints.append( QgsPointXY( 0, 0 ) );
+  mDestinationCrs = canvas->mapSettings().destinationCrs();
 
   mDialog = new QgsMeasureDialog( this );
   mDialog->setWindowFlags( mDialog->windowFlags() | Qt::Tool );
   mDialog->restorePosition();
 
-  connect( canvas, SIGNAL( destinationCrsChanged() ),
-           this, SLOT( updateSettings() ) );
+  connect( canvas, &QgsMapCanvas::destinationCrsChanged, this, &QgsMeasureTool::updateSettings );
 }
 
 QgsMeasureTool::~QgsMeasureTool()
 {
+  // important - dialog is not parented to this tool (it's parented to the main window)
+  // but we want to clean it up now
   delete mDialog;
-  delete mRubberBand;
-  delete mRubberBandPoints;
 }
 
-
-const QList<QgsPoint>& QgsMeasureTool::points()
+QVector<QgsPointXY> QgsMeasureTool::points() const
 {
   return mPoints;
 }
@@ -72,6 +68,8 @@ const QList<QgsPoint>& QgsMeasureTool::points()
 void QgsMeasureTool::activate()
 {
   mDialog->show();
+  mRubberBand->show();
+  mRubberBandPoints->show();
   QgsMapTool::activate();
 
   // ensure that we have correct settings
@@ -79,11 +77,12 @@ void QgsMeasureTool::activate()
 
   // If we suspect that they have data that is projected, yet the
   // map CRS is set to a geographic one, warn them.
-  if ( mCanvas->mapSettings().destinationCrs().geographicFlag() &&
+  if ( mCanvas->mapSettings().destinationCrs().isValid() &&
+       mCanvas->mapSettings().destinationCrs().isGeographic() &&
        ( mCanvas->extent().height() > 360 ||
          mCanvas->extent().width() > 720 ) )
   {
-    QMessageBox::warning( nullptr, tr( "Incorrect measure results" ),
+    QMessageBox::warning( nullptr, tr( "Incorrect Measure Results" ),
                           tr( "<p>This map is defined with a geographic coordinate system "
                               "(latitude/longitude) "
                               "but the map extents suggests that it is actually a projected "
@@ -98,7 +97,11 @@ void QgsMeasureTool::activate()
 
 void QgsMeasureTool::deactivate()
 {
+  mSnapIndicator->setMatch( QgsPointLocator::Match() );
+
   mDialog->hide();
+  mRubberBand->hide();
+  mRubberBandPoints->hide();
   QgsMapTool::deactivate();
 }
 
@@ -106,8 +109,8 @@ void QgsMeasureTool::restart()
 {
   mPoints.clear();
 
-  mRubberBand->reset( mMeasureArea ? QGis::Polygon : QGis::Line );
-  mRubberBandPoints->reset( QGis::Point );
+  mRubberBand->reset( mMeasureArea ? QgsWkbTypes::PolygonGeometry : QgsWkbTypes::LineGeometry );
+  mRubberBandPoints->reset( QgsWkbTypes::PointGeometry );
 
   mDone = true;
   mWrongProjectProjection = false;
@@ -115,41 +118,87 @@ void QgsMeasureTool::restart()
 
 void QgsMeasureTool::updateSettings()
 {
-  QSettings settings;
+  QgsSettings settings;
 
-  int myRed = settings.value( "/qgis/default_measure_color_red", 222 ).toInt();
-  int myGreen = settings.value( "/qgis/default_measure_color_green", 155 ).toInt();
-  int myBlue = settings.value( "/qgis/default_measure_color_blue", 67 ).toInt();
+  int myRed = settings.value( QStringLiteral( "qgis/default_measure_color_red" ), 222 ).toInt();
+  int myGreen = settings.value( QStringLiteral( "qgis/default_measure_color_green" ), 155 ).toInt();
+  int myBlue = settings.value( QStringLiteral( "qgis/default_measure_color_blue" ), 67 ).toInt();
   mRubberBand->setColor( QColor( myRed, myGreen, myBlue, 100 ) );
   mRubberBand->setWidth( 3 );
   mRubberBandPoints->setIcon( QgsRubberBand::ICON_CIRCLE );
   mRubberBandPoints->setIconSize( 10 );
   mRubberBandPoints->setColor( QColor( myRed, myGreen, myBlue, 150 ) );
+
+  // Reproject the points to the new destination CoordinateReferenceSystem
+  if ( mRubberBand->size() > 0 && mDestinationCrs != mCanvas->mapSettings().destinationCrs() && mCanvas->mapSettings().destinationCrs().isValid() )
+  {
+    QVector<QgsPointXY> points = mPoints;
+    bool lastDone = mDone;
+
+    mDialog->restart();
+    mDone = lastDone;
+    QgsCoordinateTransform ct( mDestinationCrs, mCanvas->mapSettings().destinationCrs(), QgsProject::instance() );
+
+    Q_FOREACH ( const QgsPointXY &previousPoint, points )
+    {
+      try
+      {
+        QgsPointXY point = ct.transform( previousPoint );
+
+        mPoints.append( point );
+        mRubberBand->addPoint( point, false );
+        mRubberBandPoints->addPoint( point, false );
+      }
+      catch ( QgsCsException &cse )
+      {
+        QgsMessageLog::logMessage( tr( "Transform error caught at the MeasureTool: %1" ).arg( cse.what() ) );
+      }
+    }
+
+    mRubberBand->updatePosition();
+    mRubberBand->update();
+    mRubberBandPoints->updatePosition();
+    mRubberBandPoints->update();
+  }
+  mDestinationCrs = mCanvas->mapSettings().destinationCrs();
+
   mDialog->updateSettings();
+
+  if ( !mDone && mRubberBand->size() > 0 )
+  {
+    mRubberBand->addPoint( mPoints.last() );
+    mDialog->addPoint();
+  }
+  if ( mRubberBand->size() > 0 )
+  {
+    mRubberBand->setVisible( true );
+    mRubberBandPoints->setVisible( true );
+  }
 }
 
 //////////////////////////
 
-void QgsMeasureTool::canvasPressEvent( QgsMapMouseEvent* e )
+void QgsMeasureTool::canvasPressEvent( QgsMapMouseEvent *e )
 {
   Q_UNUSED( e );
 }
 
-void QgsMeasureTool::canvasMoveEvent( QgsMapMouseEvent* e )
+void QgsMeasureTool::canvasMoveEvent( QgsMapMouseEvent *e )
 {
+  QgsPointXY point = e->snapPoint();
+  mSnapIndicator->setMatch( e->mapPointMatch() );
+
   if ( ! mDone )
   {
-    QgsPoint point = snapPoint( e->pos() );
-
     mRubberBand->movePoint( point );
     mDialog->mouseMove( point );
   }
 }
 
 
-void QgsMeasureTool::canvasReleaseEvent( QgsMapMouseEvent* e )
+void QgsMeasureTool::canvasReleaseEvent( QgsMapMouseEvent *e )
 {
-  QgsPoint point = snapPoint( e->pos() );
+  QgsPointXY point = e->snapPoint();
 
   if ( mDone ) // if we have stopped measuring any mouse click restart measuring
   {
@@ -159,14 +208,15 @@ void QgsMeasureTool::canvasReleaseEvent( QgsMapMouseEvent* e )
   if ( e->button() == Qt::RightButton ) // if we clicked the right button we stop measuring
   {
     mDone = true;
+    mRubberBand->removeLastPoint();
+    mDialog->removeLastPoint();
   }
   else if ( e->button() == Qt::LeftButton )
   {
     mDone = false;
+    addPoint( point );
   }
 
-  // we always add the clicked point to the measuring feature
-  addPoint( point );
   mDialog->show();
 
 }
@@ -175,7 +225,7 @@ void QgsMeasureTool::undo()
 {
   if ( mRubberBand )
   {
-    if ( mPoints.size() < 1 )
+    if ( mPoints.empty() )
     {
       return;
     }
@@ -199,9 +249,9 @@ void QgsMeasureTool::undo()
   }
 }
 
-void QgsMeasureTool::keyPressEvent( QKeyEvent* e )
+void QgsMeasureTool::keyPressEvent( QKeyEvent *e )
 {
-  if (( e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete ) )
+  if ( ( e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete ) )
   {
     if ( !mDone )
     {
@@ -214,7 +264,7 @@ void QgsMeasureTool::keyPressEvent( QKeyEvent* e )
 }
 
 
-void QgsMeasureTool::addPoint( const QgsPoint &point )
+void QgsMeasureTool::addPoint( const QgsPointXY &point )
 {
   QgsDebugMsg( "point=" + point.toString() );
 
@@ -224,7 +274,7 @@ void QgsMeasureTool::addPoint( const QgsPoint &point )
     return;
   }
 
-  QgsPoint pnt( point );
+  QgsPointXY pnt( point );
   // Append point that we will be moving.
   mPoints.append( pnt );
 
@@ -232,13 +282,6 @@ void QgsMeasureTool::addPoint( const QgsPoint &point )
   mRubberBandPoints->addPoint( point );
   if ( ! mDone )    // Prevent the insertion of a new item in segments measure table
   {
-    mDialog->addPoint( point );
+    mDialog->addPoint();
   }
 }
-
-
-QgsPoint QgsMeasureTool::snapPoint( const QPoint& p )
-{
-  QgsPointLocator::Match m = mCanvas->snappingUtils()->snapToMap( p );
-  return m.isValid() ? m.point() : mCanvas->getCoordinateTransform()->toMapCoordinates( p );
-}
