@@ -313,6 +313,7 @@ QgsVertexTool::~QgsVertexTool()
   delete mVertexBand;
   delete mEdgeBand;
   delete mEndpointMarker;
+  delete mVertexEditor;
 }
 
 void QgsVertexTool::activate()
@@ -1304,8 +1305,13 @@ void QgsVertexTool::onCachedGeometryChanged( QgsFeatureId fid, const QgsGeometry
   // re-run validation for the feature
   validateGeometry( layer, fid );
 
-  if ( mVertexEditor && mLockedFeature && mLockedFeature->featureId() == fid && mLockedFeature->layer() == layer )
-    mVertexEditor->updateEditor( mLockedFeature.get() );
+  if ( mLockedFeature && mLockedFeature->featureId() == fid && mLockedFeature->layer() == layer )
+  {
+    mLockedFeature->geometryChanged( fid, geom );
+    if ( mVertexEditor )
+      mVertexEditor->updateEditor( mLockedFeature.get() );
+    updateLockedFeatureVertices();
+  }
 }
 
 void QgsVertexTool::onCachedGeometryDeleted( QgsFeatureId fid )
@@ -1318,6 +1324,12 @@ void QgsVertexTool::onCachedGeometryDeleted( QgsFeatureId fid )
 
   // refresh highlighted vertices - some may have been deleted
   setHighlightedVertices( mSelectedVertices );
+
+  if ( mLockedFeature && mLockedFeature->featureId() == fid && mLockedFeature->layer() == layer )
+  {
+    mLockedFeature->featureDeleted( fid );
+    updateLockedFeatureVertices();
+  }
 }
 
 void QgsVertexTool::updateVertexEditor( QgsVectorLayer *layer, QgsFeatureId fid )
@@ -1380,12 +1392,12 @@ void QgsVertexTool::showVertexEditor()  //#spellok
 {
   if ( !mVertexEditor )
   {
-    mVertexEditor.reset( new QgsVertexEditor( mCanvas ) );
-    if ( !QgisApp::instance()->restoreDockWidget( mVertexEditor.get() ) )
-      QgisApp::instance()->addDockWidget( Qt::LeftDockWidgetArea, mVertexEditor.get() );
+    mVertexEditor = new QgsVertexEditor( mCanvas );
+    if ( !QgisApp::instance()->restoreDockWidget( mVertexEditor ) )
+      QgisApp::instance()->addDockWidget( Qt::LeftDockWidgetArea, mVertexEditor );
 
-    connect( mVertexEditor.get(), &QgsVertexEditor::deleteSelectedRequested, this, &QgsVertexTool::deleteVertexEditorSelection );
-    connect( mVertexEditor.get(), &QgsVertexEditor::editorClosed, this, &QgsVertexTool::cleanupVertexEditor );
+    connect( mVertexEditor, &QgsVertexEditor::deleteSelectedRequested, this, &QgsVertexTool::deleteVertexEditorSelection );
+    connect( mVertexEditor, &QgsVertexEditor::editorClosed, this, &QgsVertexTool::cleanupVertexEditor );
 
     // timer required as showing/raising the vertex editor in the same function following restoreDockWidget fails
     QTimer::singleShot( 200, this, [ = ] { mVertexEditor->show(); mVertexEditor->raise(); } );
@@ -1400,7 +1412,11 @@ void QgsVertexTool::showVertexEditor()  //#spellok
 void QgsVertexTool::cleanupVertexEditor()
 {
   mLockedFeature.reset();
-  mVertexEditor.reset();
+  // do not delete immediately because vertex editor
+  // can be still used in the qt event loop
+  mVertexEditor->deleteLater();
+
+  updateLockedFeatureVertices();
 }
 
 void QgsVertexTool::cleanupLockedFeature()
@@ -2022,6 +2038,7 @@ void QgsVertexTool::moveVertex( const QgsPointXY &mapPoint, const QgsPointLocato
     }
   }
 
+  updateLockedFeatureVertices();
   if ( mVertexEditor )
     mVertexEditor->updateEditor( mLockedFeature.get() );
 
