@@ -21,7 +21,6 @@
 #include <cmath>
 
 #include <QDir>
-#include <QTemporaryFile>
 #include <QDomNode>
 #include <QDomElement>
 #include <QFileInfo>
@@ -638,8 +637,10 @@ bool QgsCoordinateReferenceSystem::createFromWkt( const QString &wkt )
   {
     CPLFree( proj4src );
 
+#if GDAL_VERSION_NUM < GDAL_COMPUTE_VERSION(2,5,0)
     // try fixed up version
     OSRFixup( d->mCRS );
+#endif
 
     OSRExportToProj4( d->mCRS, &proj4src );
 
@@ -1176,9 +1177,11 @@ void QgsCoordinateReferenceSystem::setMapUnits()
 
   char *unitName = nullptr;
 
+#if GDAL_VERSION_NUM < GDAL_COMPUTE_VERSION(2,5,0)
   // Of interest to us is that this call adds in a unit parameter if
   // one doesn't already exist.
   OSRFixup( d->mCRS );
+#endif
 
   if ( OSRIsProjected( d->mCRS ) )
   {
@@ -1770,7 +1773,12 @@ bool QgsCoordinateReferenceSystem::loadIds( QHash<int, QString> &wkts )
 
   Q_FOREACH ( const QString &csv, QStringList() << "gcs.csv" << "pcs.csv" << "vertcs.csv" << "compdcs.csv" << "geoccs.csv" )
   {
-    QString filename = CPLFindFile( "gdal", csv.toUtf8() );
+
+#if GDAL_VERSION_NUM < GDAL_COMPUTE_VERSION(3,0,0)
+    const QString filename = CPLFindFile( "gdal", csv.toUtf8() );
+#else
+    const QString filename = QgsApplication::pkgDataPath() + QStringLiteral( "/resources/" ) + csv;
+#endif
 
     QFile f( filename );
     if ( !f.open( QIODevice::ReadOnly ) )
@@ -2156,8 +2164,13 @@ int QgsCoordinateReferenceSystem::syncDatabase()
 
 bool QgsCoordinateReferenceSystem::syncDatumTransform( const QString &dbPath )
 {
+#if GDAL_VERSION_NUM < GDAL_COMPUTE_VERSION(3,0,0)
   const char *filename = CSVFilename( "datum_shift.csv" );
   FILE *fp = VSIFOpen( filename, "rb" );
+#else
+  const QString filename = QgsApplication::pkgDataPath() + QStringLiteral( "/resources/datum_shift.csv" );
+  FILE *fp = VSIFOpen( filename.toUtf8().constData(), "rb" );
+#endif
   if ( !fp )
   {
     return false;
