@@ -32,24 +32,15 @@
 #include <QImage>
 #include <QPainter>
 
-QgsRasterResampleFilter::QgsRasterResampleFilter( QgsRasterInterface* input )
-    : QgsRasterInterface( input )
-    , mZoomedInResampler( nullptr )
-    , mZoomedOutResampler( nullptr )
-    , mMaxOversampling( 2.0 )
+QgsRasterResampleFilter::QgsRasterResampleFilter( QgsRasterInterface *input )
+  : QgsRasterInterface( input )
 {
 }
 
-QgsRasterResampleFilter::~QgsRasterResampleFilter()
+QgsRasterResampleFilter *QgsRasterResampleFilter::clone() const
 {
-  delete mZoomedInResampler;
-  delete mZoomedOutResampler;
-}
-
-QgsRasterResampleFilter* QgsRasterResampleFilter::clone() const
-{
-  QgsDebugMsgLevel( "Entered", 4 );
-  QgsRasterResampleFilter * resampler = new QgsRasterResampleFilter( nullptr );
+  QgsDebugMsgLevel( QStringLiteral( "Entered" ), 4 );
+  QgsRasterResampleFilter *resampler = new QgsRasterResampleFilter( nullptr );
   if ( mZoomedInResampler )
   {
     resampler->setZoomedInResampler( mZoomedInResampler->clone() );
@@ -71,88 +62,82 @@ int QgsRasterResampleFilter::bandCount() const
   return 0;
 }
 
-QGis::DataType QgsRasterResampleFilter::dataType( int bandNo ) const
+Qgis::DataType QgsRasterResampleFilter::dataType( int bandNo ) const
 {
-  if ( mOn ) return QGis::ARGB32_Premultiplied;
+  if ( mOn ) return Qgis::ARGB32_Premultiplied;
 
   if ( mInput ) return mInput->dataType( bandNo );
 
-  return QGis::UnknownDataType;
+  return Qgis::UnknownDataType;
 }
 
-bool QgsRasterResampleFilter::setInput( QgsRasterInterface* input )
+bool QgsRasterResampleFilter::setInput( QgsRasterInterface *input )
 {
-  QgsDebugMsgLevel( "Entered", 4 );
+  QgsDebugMsgLevel( QStringLiteral( "Entered" ), 4 );
 
   // Resampler can only work with single band ARGB32_Premultiplied
   if ( !input )
   {
-    QgsDebugMsg( "No input" );
+    QgsDebugMsg( QStringLiteral( "No input" ) );
     return false;
   }
 
   if ( !mOn )
   {
     // In off mode we can connect to anything
-    QgsDebugMsgLevel( "OK", 4 );
+    QgsDebugMsgLevel( QStringLiteral( "OK" ), 4 );
     mInput = input;
     return true;
   }
 
   if ( input->bandCount() < 1 )
   {
-    QgsDebugMsg( "No input band" );
+    QgsDebugMsg( QStringLiteral( "No input band" ) );
     return false;
   }
 
-  if ( input->dataType( 1 ) != QGis::ARGB32_Premultiplied &&
-       input->dataType( 1 ) != QGis::ARGB32 )
+  if ( input->dataType( 1 ) != Qgis::ARGB32_Premultiplied &&
+       input->dataType( 1 ) != Qgis::ARGB32 )
   {
-    QgsDebugMsg( "Unknown input data type" );
+    QgsDebugMsg( QStringLiteral( "Unknown input data type" ) );
     return false;
   }
 
   mInput = input;
-  QgsDebugMsgLevel( "OK", 4 );
+  QgsDebugMsgLevel( QStringLiteral( "OK" ), 4 );
   return true;
 }
 
-void QgsRasterResampleFilter::setZoomedInResampler( QgsRasterResampler* r )
+void QgsRasterResampleFilter::setZoomedInResampler( QgsRasterResampler *r )
 {
-  delete mZoomedInResampler;
-  mZoomedInResampler = r;
+  mZoomedInResampler.reset( r );
 }
 
-void QgsRasterResampleFilter::setZoomedOutResampler( QgsRasterResampler* r )
+void QgsRasterResampleFilter::setZoomedOutResampler( QgsRasterResampler *r )
 {
-  delete mZoomedOutResampler;
-  mZoomedOutResampler = r;
+  mZoomedOutResampler.reset( r );
 }
 
-QgsRasterBlock * QgsRasterResampleFilter::block( int bandNo, QgsRectangle  const & extent, int width, int height )
-{
-  return block2( bandNo, extent, width, height );
-}
-
-QgsRasterBlock * QgsRasterResampleFilter::block2( int bandNo, QgsRectangle  const & extent, int width, int height, QgsRasterBlockFeedback* feedback )
+QgsRasterBlock *QgsRasterResampleFilter::block( int bandNo, QgsRectangle  const &extent, int width, int height, QgsRasterBlockFeedback *feedback )
 {
   Q_UNUSED( bandNo );
-  QgsDebugMsgLevel( QString( "width = %1 height = %2 extent = %3" ).arg( width ).arg( height ).arg( extent.toString() ), 4 );
-  QgsRasterBlock *outputBlock = new QgsRasterBlock();
-  if ( !mInput ) return outputBlock;
+  QgsDebugMsgLevel( QStringLiteral( "width = %1 height = %2 extent = %3" ).arg( width ).arg( height ).arg( extent.toString() ), 4 );
+  std::unique_ptr< QgsRasterBlock > outputBlock( new QgsRasterBlock() );
+  if ( !mInput )
+    return outputBlock.release();
 
   double oversampling = 1.0; // approximate global oversampling factor
 
   if ( mZoomedInResampler || mZoomedOutResampler )
   {
-    QgsRasterDataProvider *provider = dynamic_cast<QgsRasterDataProvider*>( mInput->srcInput() );
+    QgsRasterDataProvider *provider = dynamic_cast<QgsRasterDataProvider *>( mInput->sourceInput() );
     if ( provider && ( provider->capabilities() & QgsRasterDataProvider::Size ) )
     {
       double xRes = extent.width() / width;
       double providerXRes = provider->extent().width() / provider->xSize();
       double pixelRatio = xRes / providerXRes;
       oversampling = ( pixelRatio > mMaxOversampling ) ? mMaxOversampling : pixelRatio;
-      QgsDebugMsgLevel( QString( "xRes = %1 providerXRes = %2 pixelRatio = %3 oversampling = %4" ).arg( xRes ).arg( providerXRes ).arg( pixelRatio ).arg( oversampling ), 4 );
+      QgsDebugMsgLevel( QStringLiteral( "xRes = %1 providerXRes = %2 pixelRatio = %3 oversampling = %4" ).arg( xRes ).arg( providerXRes ).arg( pixelRatio ).arg( oversampling ), 4 );
     }
     else
     {
@@ -163,18 +148,17 @@ QgsRasterBlock * QgsRasterResampleFilter::block2( int bandNo, QgsRectangle  cons
     }
   }
 
-  QgsDebugMsgLevel( QString( "oversampling %1" ).arg( oversampling ), 4 );
+  QgsDebugMsgLevel( QStringLiteral( "oversampling %1" ).arg( oversampling ), 4 );
 
   int bandNumber = 1;
 
   // Do no oversampling if no resampler for zoomed in / zoomed out (nearest neighbour)
   // We do mZoomedInResampler if oversampling == 1 (otherwise for example reprojected
   // zoom in rasters are never resampled because projector limits resolution.
-  if ((( oversampling < 1.0 || qgsDoubleNear( oversampling, 1.0 ) ) && !mZoomedInResampler ) || ( oversampling > 1.0 && !mZoomedOutResampler ) )
+  if ( ( ( oversampling < 1.0 || qgsDoubleNear( oversampling, 1.0 ) ) && !mZoomedInResampler ) || ( oversampling > 1.0 && !mZoomedOutResampler ) )
   {
-    QgsDebugMsgLevel( "No oversampling.", 4 );
-    delete outputBlock;
-    return mInput->block2( bandNumber, extent, width, height, feedback );
+    QgsDebugMsgLevel( QStringLiteral( "No oversampling." ), 4 );
+    return mInput->block( bandNumber, extent, width, height, feedback );
   }
 
   //effective oversampling factors are different to global one because of rounding
@@ -186,18 +170,16 @@ QgsRasterBlock * QgsRasterResampleFilter::block2( int bandNo, QgsRectangle  cons
   int resWidth = width * oversamplingX;
   int resHeight = height * oversamplingY;
 
-  QgsRasterBlock *inputBlock = mInput->block2( bandNumber, extent, resWidth, resHeight, feedback );
+  std::unique_ptr< QgsRasterBlock > inputBlock( mInput->block( bandNumber, extent, resWidth, resHeight, feedback ) );
   if ( !inputBlock || inputBlock->isEmpty() )
   {
-    QgsDebugMsg( "No raster data!" );
-    delete inputBlock;
-    return outputBlock;
+    QgsDebugMsg( QStringLiteral( "No raster data!" ) );
+    return outputBlock.release();
   }
 
-  if ( !outputBlock->reset( QGis::ARGB32_Premultiplied, width, height ) )
+  if ( !outputBlock->reset( Qgis::ARGB32_Premultiplied, width, height ) )
   {
-    delete inputBlock;
-    return outputBlock;
+    return outputBlock.release();
   }
 
   //resample image
@@ -207,70 +189,69 @@ QgsRasterBlock * QgsRasterResampleFilter::block2( int bandNo, QgsRectangle  cons
 
   if ( mZoomedInResampler && ( oversamplingX < 1.0 || qgsDoubleNear( oversampling, 1.0 ) ) )
   {
-    QgsDebugMsgLevel( "zoomed in resampling", 4 );
+    QgsDebugMsgLevel( QStringLiteral( "zoomed in resampling" ), 4 );
     mZoomedInResampler->resample( img, dstImg );
   }
   else if ( mZoomedOutResampler && oversamplingX > 1.0 )
   {
-    QgsDebugMsgLevel( "zoomed out resampling", 4 );
+    QgsDebugMsgLevel( QStringLiteral( "zoomed out resampling" ), 4 );
     mZoomedOutResampler->resample( img, dstImg );
   }
   else
   {
     // Should not happen
-    QgsDebugMsg( "Unexpected resampling" );
+    QgsDebugMsg( QStringLiteral( "Unexpected resampling" ) );
     dstImg = img.scaled( width, height );
   }
 
   outputBlock->setImage( &dstImg );
 
-  delete inputBlock;
-  return outputBlock; // No resampling
+  return outputBlock.release(); // No resampling
 }
 
-void QgsRasterResampleFilter::writeXML( QDomDocument& doc, QDomElement& parentElem ) const
+void QgsRasterResampleFilter::writeXml( QDomDocument &doc, QDomElement &parentElem ) const
 {
   if ( parentElem.isNull() )
   {
     return;
   }
 
-  QDomElement rasterRendererElem = doc.createElement( "rasterresampler" );
+  QDomElement rasterRendererElem = doc.createElement( QStringLiteral( "rasterresampler" ) );
 
-  rasterRendererElem.setAttribute( "maxOversampling", QString::number( mMaxOversampling ) );
+  rasterRendererElem.setAttribute( QStringLiteral( "maxOversampling" ), QString::number( mMaxOversampling ) );
   if ( mZoomedInResampler )
   {
-    rasterRendererElem.setAttribute( "zoomedInResampler", mZoomedInResampler->type() );
+    rasterRendererElem.setAttribute( QStringLiteral( "zoomedInResampler" ), mZoomedInResampler->type() );
   }
   if ( mZoomedOutResampler )
   {
-    rasterRendererElem.setAttribute( "zoomedOutResampler", mZoomedOutResampler->type() );
+    rasterRendererElem.setAttribute( QStringLiteral( "zoomedOutResampler" ), mZoomedOutResampler->type() );
   }
   parentElem.appendChild( rasterRendererElem );
 }
 
-void QgsRasterResampleFilter::readXML( const QDomElement& filterElem )
+void QgsRasterResampleFilter::readXml( const QDomElement &filterElem )
 {
   if ( filterElem.isNull() )
   {
     return;
   }
 
-  mMaxOversampling = filterElem.attribute( "maxOversampling", "2.0" ).toDouble();
+  mMaxOversampling = filterElem.attribute( QStringLiteral( "maxOversampling" ), QStringLiteral( "2.0" ) ).toDouble();
 
-  QString zoomedInResamplerType = filterElem.attribute( "zoomedInResampler" );
-  if ( zoomedInResamplerType == "bilinear" )
+  QString zoomedInResamplerType = filterElem.attribute( QStringLiteral( "zoomedInResampler" ) );
+  if ( zoomedInResamplerType == QLatin1String( "bilinear" ) )
   {
-    mZoomedInResampler = new QgsBilinearRasterResampler();
+    mZoomedInResampler.reset( new QgsBilinearRasterResampler() );
   }
-  else if ( zoomedInResamplerType == "cubic" )
+  else if ( zoomedInResamplerType == QLatin1String( "cubic" ) )
   {
-    mZoomedInResampler = new QgsCubicRasterResampler();
+    mZoomedInResampler.reset( new QgsCubicRasterResampler() );
   }
 
-  QString zoomedOutResamplerType = filterElem.attribute( "zoomedOutResampler" );
-  if ( zoomedOutResamplerType == "bilinear" )
+  QString zoomedOutResamplerType = filterElem.attribute( QStringLiteral( "zoomedOutResampler" ) );
+  if ( zoomedOutResamplerType == QLatin1String( "bilinear" ) )
   {
-    mZoomedOutResampler = new QgsBilinearRasterResampler();
+    mZoomedOutResampler.reset( new QgsBilinearRasterResampler() );
   }
 }
