@@ -25,6 +25,7 @@
 
 #include "qgsapplication.h"
 #include "qgscoordinatetransform.h"
+#include "qgshtmlutils.h"
 #include "qgsrectangle.h"
 #include "qgscoordinatereferencesystem.h"
 #include "qgsrasterbandstats.h"
@@ -41,16 +42,11 @@
 #define ERR(message) QGS_ERROR_MESSAGE(message,"GRASS provider")
 #define QGS_ERROR(message) QgsError(message,"GRASS provider")
 
-// Do not use warning dialogs, providers are also created on threads (rendering) where dialogs connot be used (constructing QPixmap icon)
+// Do not use warning dialogs, providers are also created on threads (rendering) where dialogs cannot be used (constructing QPixmap icon)
 
-QgsGrassRasterProvider::QgsGrassRasterProvider( QString const & uri )
-    : QgsRasterDataProvider( uri )
-    , mValid( false )
-    , mGrassDataType( 0 )
-    , mCols( 0 )
-    , mRows( 0 )
-    , mYBlockSize( 0 )
-    , mNoDataValue( std::numeric_limits<double>::quiet_NaN() )
+QgsGrassRasterProvider::QgsGrassRasterProvider( QString const &uri )
+  : QgsRasterDataProvider( uri )
+  , mNoDataValue( std::numeric_limits<double>::quiet_NaN() )
 {
   QgsDebugMsg( "QgsGrassRasterProvider: constructing with uri '" + uri + "'." );
 
@@ -71,7 +67,7 @@ QgsGrassRasterProvider::QgsGrassRasterProvider( QString const & uri )
   mMapName = fileInfo.fileName();
   QDir dir = fileInfo.dir();
   QString element = dir.dirName();
-  if ( element != "cellhd" )
+  if ( element != QLatin1String( "cellhd" ) )
   {
     appendError( ERR( tr( "Groups not yet supported" ) ) );
     return;
@@ -106,10 +102,10 @@ QgsGrassRasterProvider::QgsGrassRasterProvider( QString const & uri )
 
   error.clear();
   mInfo = QgsGrass::info( mGisdbase, mLocation, mMapset, mMapName, QgsGrassObject::Raster,
-                          "info", QgsRectangle(), 0, 0, 3000, error );
+                          QStringLiteral( "info" ), QgsRectangle(), 0, 0, 3000, error );
   appendIfError( error );
 
-  mGrassDataType = mInfo["TYPE"].toInt();
+  mGrassDataType = mInfo[QStringLiteral( "TYPE" )].toInt();
   QgsDebugMsg( "mGrassDataType = " + QString::number( mGrassDataType ) );
 
   // TODO: avoid showing these strange numbers in GUI
@@ -118,12 +114,12 @@ QgsGrassRasterProvider::QgsGrassRasterProvider( QString const & uri )
   double myInternalNoDataValue;
   if ( mGrassDataType == CELL_TYPE )
   {
-    myInternalNoDataValue = INT_MIN;
+    myInternalNoDataValue = std::numeric_limits<int>::min();
   }
   else if ( mGrassDataType == DCELL_TYPE )
   {
     // Don't use numeric limits, raster layer is using
-    //    qAbs( myValue - mNoDataValue ) <= TINY_VALUE
+    //    std::fabs( myValue - mNoDataValue ) <= TINY_VALUE
     // if the mNoDataValue would be a limit, the subtraction could overflow.
     // No data value is shown in GUI, use some nice number.
     // Choose values with small representation error.
@@ -171,58 +167,14 @@ QgsGrassRasterProvider::~QgsGrassRasterProvider()
   QgsDebugMsg( "QgsGrassRasterProvider: deconstructing." );
 }
 
-QgsRasterInterface * QgsGrassRasterProvider::clone() const
+QgsRasterInterface *QgsGrassRasterProvider::clone() const
 {
-  QgsGrassRasterProvider * provider = new QgsGrassRasterProvider( dataSourceUri() );
+  QgsGrassRasterProvider *provider = new QgsGrassRasterProvider( dataSourceUri() );
   provider->copyBaseSettings( *this );
   return provider;
 }
 
-QImage* QgsGrassRasterProvider::draw( QgsRectangle  const & viewExtent, int pixelWidth, int pixelHeight )
-{
-  QgsDebugMsg( "pixelWidth = "  + QString::number( pixelWidth ) );
-  QgsDebugMsg( "pixelHeight = "  + QString::number( pixelHeight ) );
-  QgsDebugMsg( "viewExtent: " + viewExtent.toString() );
-  clearLastError();
-
-  QImage *image = new QImage( pixelWidth, pixelHeight, QImage::Format_ARGB32 );
-  image->fill( QColor( Qt::gray ).rgb() );
-
-  QStringList arguments;
-  arguments.append( "map=" +  mMapName + "@" + mMapset );
-
-  arguments.append(( QString( "window=%1,%2,%3,%4,%5,%6" )
-                     .arg( QgsRasterBlock::printValue( viewExtent.xMinimum() ),
-                           QgsRasterBlock::printValue( viewExtent.yMinimum() ),
-                           QgsRasterBlock::printValue( viewExtent.xMaximum() ),
-                           QgsRasterBlock::printValue( viewExtent.yMaximum() ) )
-                     .arg( pixelWidth ).arg( pixelHeight ) ) );
-  QString cmd = QgsApplication::libexecPath() + "grass/modules/qgis.d.rast";
-  QByteArray data;
-  try
-  {
-    data = QgsGrass::runModule( mGisdbase, mLocation, mMapset, cmd, arguments );
-  }
-  catch ( QgsGrass::Exception &e )
-  {
-    QString error = tr( "Cannot draw raster" ) + " : " + e.what();
-    QgsDebugMsg( error );
-    appendError( error );
-    // We don't set mValid to false, because the raster can be recreated and work next time
-    return image;
-  }
-  QgsDebugMsg( QString( "%1 bytes read from modules stdout" ).arg( data.size() ) );
-  uchar * ptr = image->bits();
-  // byteCount() in Qt >= 4.6
-  //int size = image->byteCount() < data.size() ? image->byteCount() : data.size();
-  int size = pixelWidth * pixelHeight * 4 < data.size() ? pixelWidth * pixelHeight * 4 : data.size();
-  memcpy( ptr, data.data(), size );
-
-  return image;
-}
-
-
-void QgsGrassRasterProvider::readBlock( int bandNo, int xBlock, int yBlock, void *block )
+bool QgsGrassRasterProvider::readBlock( int bandNo, int xBlock, int yBlock, void *block )
 {
   Q_UNUSED( xBlock );
   clearLastError();
@@ -242,14 +194,14 @@ void QgsGrassRasterProvider::readBlock( int bandNo, int xBlock, int yBlock, void
   double yMinimum = yMaximum - cellHeight * mYBlockSize;
 
   QgsDebugMsg( "mYBlockSize = " + QString::number( mYBlockSize ) );
-  arguments.append(( QString( "window=%1,%2,%3,%4,%5,%6" )
-                     .arg( QgsRasterBlock::printValue( ext.xMinimum() ),
-                           QgsRasterBlock::printValue( yMinimum ),
-                           QgsRasterBlock::printValue( ext.xMaximum() ),
-                           QgsRasterBlock::printValue( yMaximum ) )
-                     .arg( mCols ).arg( mYBlockSize ) ) );
+  arguments.append( ( QStringLiteral( "window=%1,%2,%3,%4,%5,%6" )
+                      .arg( QgsRasterBlock::printValue( ext.xMinimum() ),
+                            QgsRasterBlock::printValue( yMinimum ),
+                            QgsRasterBlock::printValue( ext.xMaximum() ),
+                            QgsRasterBlock::printValue( yMaximum ) )
+                      .arg( mCols ).arg( mYBlockSize ) ) );
 
-  arguments.append( "format=value" );
+  arguments.append( QStringLiteral( "format=value" ) );
   QString cmd = QgsApplication::libexecPath() + "grass/modules/qgis.d.rast";
   QByteArray data;
   try
@@ -262,6 +214,7 @@ void QgsGrassRasterProvider::readBlock( int bandNo, int xBlock, int yBlock, void
     QgsDebugMsg( error );
     appendError( error );
     // We don't set mValid to false, because the raster can be recreated and work next time
+    return false;
   }
   QgsDebugMsg( QString( "%1 bytes read from modules stdout" ).arg( data.size() ) );
   // byteCount() in Qt >= 4.6
@@ -276,9 +229,11 @@ void QgsGrassRasterProvider::readBlock( int bandNo, int xBlock, int yBlock, void
     size = size < data.size() ? size : data.size();
   }
   memcpy( block, data.data(), size );
+
+  return true;
 }
 
-void QgsGrassRasterProvider::readBlock( int bandNo, QgsRectangle  const & viewExtent, int pixelWidth, int pixelHeight, void *block, QgsRasterBlockFeedback* feedback )
+bool QgsGrassRasterProvider::readBlock( int bandNo, QgsRectangle  const &viewExtent, int pixelWidth, int pixelHeight, void *block, QgsRasterBlockFeedback *feedback )
 {
   Q_UNUSED( feedback );
   QgsDebugMsg( "pixelWidth = "  + QString::number( pixelWidth ) );
@@ -287,18 +242,18 @@ void QgsGrassRasterProvider::readBlock( int bandNo, QgsRectangle  const & viewEx
   clearLastError();
 
   if ( pixelWidth <= 0 || pixelHeight <= 0 )
-    return;
+    return false;
 
   QStringList arguments;
   arguments.append( "map=" +  mMapName + "@" + mMapset );
 
-  arguments.append(( QString( "window=%1,%2,%3,%4,%5,%6" )
-                     .arg( QgsRasterBlock::printValue( viewExtent.xMinimum() ),
-                           QgsRasterBlock::printValue( viewExtent.yMinimum() ),
-                           QgsRasterBlock::printValue( viewExtent.xMaximum() ),
-                           QgsRasterBlock::printValue( viewExtent.yMaximum() ) )
-                     .arg( pixelWidth ).arg( pixelHeight ) ) );
-  arguments.append( "format=value" );
+  arguments.append( ( QStringLiteral( "window=%1,%2,%3,%4,%5,%6" )
+                      .arg( QgsRasterBlock::printValue( viewExtent.xMinimum() ),
+                            QgsRasterBlock::printValue( viewExtent.yMinimum() ),
+                            QgsRasterBlock::printValue( viewExtent.xMaximum() ),
+                            QgsRasterBlock::printValue( viewExtent.yMaximum() ) )
+                      .arg( pixelWidth ).arg( pixelHeight ) ) );
+  arguments.append( QStringLiteral( "format=value" ) );
   QString cmd = QgsApplication::libexecPath() + "grass/modules/qgis.d.rast";
   QByteArray data;
   try
@@ -312,7 +267,7 @@ void QgsGrassRasterProvider::readBlock( int bandNo, QgsRectangle  const & viewEx
     appendError( error );
 
     // We don't set mValid to false, because the raster can be recreated and work next time
-    return;
+    return false;
   }
   QgsDebugMsg( QString( "%1 bytes read from modules stdout" ).arg( data.size() ) );
   // byteCount() in Qt >= 4.6
@@ -326,15 +281,17 @@ void QgsGrassRasterProvider::readBlock( int bandNo, QgsRectangle  const & viewEx
     size = size < data.size() ? size : data.size();
   }
   memcpy( block, data.data(), size );
+
+  return true;
 }
 
-QgsRasterBandStats QgsGrassRasterProvider::bandStatistics( int theBandNo, int theStats, const QgsRectangle & theExtent, int theSampleSize )
+QgsRasterBandStats QgsGrassRasterProvider::bandStatistics( int bandNo, int stats, const QgsRectangle &boundingBox, int sampleSize, QgsRasterBlockFeedback * )
 {
-  QgsDebugMsg( QString( "theBandNo = %1 theSampleSize = %2" ).arg( theBandNo ).arg( theSampleSize ) );
+  QgsDebugMsg( QString( "theBandNo = %1 sampleSize = %2" ).arg( bandNo ).arg( sampleSize ) );
   QgsRasterBandStats myRasterBandStats;
-  initStatistics( myRasterBandStats, theBandNo, theStats, theExtent, theSampleSize );
+  initStatistics( myRasterBandStats, bandNo, stats, boundingBox, sampleSize );
 
-  Q_FOREACH ( const QgsRasterBandStats& stats, mStatistics )
+  Q_FOREACH ( const QgsRasterBandStats &stats, mStatistics )
   {
     if ( stats.contains( myRasterBandStats ) )
     {
@@ -354,21 +311,21 @@ QgsRasterBandStats QgsGrassRasterProvider::bandStatistics( int theBandNo, int th
 
   QString error;
   QHash<QString, QString> info = QgsGrass::info( mGisdbase, mLocation, mMapset, mMapName, QgsGrassObject::Raster,
-                                 "stats", extent, sampleRows, sampleCols, timeout, error );
+                                 QStringLiteral( "stats" ), extent, sampleRows, sampleCols, timeout, error );
 
   if ( info.isEmpty() || !error.isEmpty() )
   {
     return myRasterBandStats;
   }
 
-  myRasterBandStats.sum = info["SUM"].toDouble();
-  myRasterBandStats.elementCount = info["COUNT"].toInt();
-  myRasterBandStats.minimumValue = info["MIN"].toDouble();
-  myRasterBandStats.maximumValue = info["MAX"].toDouble();
+  myRasterBandStats.sum = info[QStringLiteral( "SUM" )].toDouble();
+  myRasterBandStats.elementCount = info[QStringLiteral( "COUNT" )].toInt();
+  myRasterBandStats.minimumValue = info[QStringLiteral( "MIN" )].toDouble();
+  myRasterBandStats.maximumValue = info[QStringLiteral( "MAX" )].toDouble();
   myRasterBandStats.range = myRasterBandStats.maximumValue - myRasterBandStats.minimumValue;
-  myRasterBandStats.sumOfSquares = info["SQSUM"].toDouble();
-  myRasterBandStats.mean = info["MEAN"].toDouble();
-  myRasterBandStats.stdDev = info["STDEV"].toDouble();
+  myRasterBandStats.sumOfSquares = info[QStringLiteral( "SQSUM" )].toDouble();
+  myRasterBandStats.mean = info[QStringLiteral( "MEAN" )].toDouble();
+  myRasterBandStats.stdDev = info[QStringLiteral( "STDEV" )].toDouble();
 
   QgsDebugMsg( QString( "min = %1" ).arg( myRasterBandStats.minimumValue ) );
   QgsDebugMsg( QString( "max = %1" ).arg( myRasterBandStats.maximumValue ) );
@@ -389,7 +346,7 @@ QList<QgsColorRampShader::ColorRampItem> QgsGrassRasterProvider::colorTable( int
   Q_UNUSED( bandNo );
   QList<QgsColorRampShader::ColorRampItem> ct;
 
-  // TODO: check if color can be realy discontinuous in GRASS,
+  // TODO: check if color can be really discontinuous in GRASS,
   // for now we just believe that they are continuous, i.e. end and beginning
   // of the ramp with the same value has the same color
   // we are also expecting ordered CT records in the list
@@ -427,12 +384,12 @@ QList<QgsColorRampShader::ColorRampItem> QgsGrassRasterProvider::colorTable( int
   return ct;
 }
 
-QgsCoordinateReferenceSystem QgsGrassRasterProvider::crs()
+QgsCoordinateReferenceSystem QgsGrassRasterProvider::crs() const
 {
   return mCrs;
 }
 
-QgsRectangle QgsGrassRasterProvider::extent()
+QgsRectangle QgsGrassRasterProvider::extent() const
 {
   // The extend can change of course so we get always fresh, to avoid running always the module
   // we should save mExtent and mLastModified and check if the map was modified
@@ -456,22 +413,22 @@ int QgsGrassRasterProvider::yBlockSize() const
 int QgsGrassRasterProvider::xSize() const { return mCols; }
 int QgsGrassRasterProvider::ySize() const { return mRows; }
 
-QgsRasterIdentifyResult QgsGrassRasterProvider::identify( const QgsPoint & thePoint, QgsRaster::IdentifyFormat theFormat, const QgsRectangle &theExtent, int theWidth, int theHeight, int /*theDpi*/ )
+QgsRasterIdentifyResult QgsGrassRasterProvider::identify( const QgsPointXY &point, QgsRaster::IdentifyFormat format, const QgsRectangle &boundingBox, int width, int height, int /*dpi*/ )
 {
-  Q_UNUSED( theExtent );
-  Q_UNUSED( theWidth );
-  Q_UNUSED( theHeight );
+  Q_UNUSED( boundingBox );
+  Q_UNUSED( width );
+  Q_UNUSED( height );
   QMap<int, QVariant> results;
   QMap<int, QVariant> noDataResults;
   noDataResults.insert( 1, QVariant() );
   QgsRasterIdentifyResult noDataResult( QgsRaster::IdentifyFormatValue, results );
 
-  if ( theFormat != QgsRaster::IdentifyFormatValue )
+  if ( format != QgsRaster::IdentifyFormatValue )
   {
     return QgsRasterIdentifyResult( QGS_ERROR( tr( "Format not supported" ) ) );
   }
 
-  if ( !extent().contains( thePoint ) )
+  if ( !extent().contains( point ) )
   {
     return noDataResult;
   }
@@ -481,7 +438,7 @@ QgsRasterIdentifyResult QgsGrassRasterProvider::identify( const QgsPoint & thePo
   // attention, value tool does his own tricks with grass identify() so it stops to refresh values outside extent or null values e.g.
 
   bool ok;
-  double value = mRasterValue.value( thePoint.x(), thePoint.y(), &ok );
+  double value = mRasterValue.value( point.x(), point.y(), &ok );
 
   if ( !ok )
   {
@@ -489,7 +446,7 @@ QgsRasterIdentifyResult QgsGrassRasterProvider::identify( const QgsPoint & thePo
   }
 
   // no data?
-  if ( qIsNaN( value ) || qgsDoubleNear( value, mNoDataValue ) )
+  if ( std::isnan( value ) || qgsDoubleNear( value, mNoDataValue ) )
   {
     return noDataResult;
   }
@@ -514,24 +471,24 @@ int QgsGrassRasterProvider::capabilities() const
   return capability;
 }
 
-QGis::DataType QgsGrassRasterProvider::dataType( int bandNo ) const
+Qgis::DataType QgsGrassRasterProvider::dataType( int bandNo ) const
 {
-  return srcDataType( bandNo );
+  return sourceDataType( bandNo );
 }
 
-QGis::DataType QgsGrassRasterProvider::srcDataType( int bandNo ) const
+Qgis::DataType QgsGrassRasterProvider::sourceDataType( int bandNo ) const
 {
   Q_UNUSED( bandNo );
   switch ( mGrassDataType )
   {
     case CELL_TYPE:
-      return QGis::Int32;
+      return Qgis::Int32;
     case FCELL_TYPE:
-      return QGis::Float32;
+      return Qgis::Float32;
     case DCELL_TYPE:
-      return QGis::Float64;
+      return Qgis::Float64;
   }
-  return QGis::UnknownDataType;
+  return Qgis::UnknownDataType;
 }
 
 int QgsGrassRasterProvider::bandCount() const
@@ -551,7 +508,7 @@ int QgsGrassRasterProvider::colorInterpretation( int bandNo ) const
   return QgsRaster::GrayIndex;
 }
 
-QString QgsGrassRasterProvider::metadata()
+QString QgsGrassRasterProvider::htmlMetadata()
 {
   QString myMetadata;
   QStringList myList;
@@ -565,18 +522,16 @@ QString QgsGrassRasterProvider::metadata()
   {
     myList.append( i.key() + " : " + i.value() );
   }
-  myMetadata += QgsRasterDataProvider::makeTableCells( myList );
-
-
+  myMetadata += QgsHtmlUtils::buildBulletList( myList );
   return myMetadata;
 }
 
-bool QgsGrassRasterProvider::isValid()
+bool QgsGrassRasterProvider::isValid() const
 {
   return mValid;
 }
 
-void QgsGrassRasterProvider::setLastError( QString error )
+void QgsGrassRasterProvider::setLastError( const QString &error )
 {
   mLastErrorTitle = tr( "GRASS raster provider" );
   mLastError = error;
@@ -588,7 +543,7 @@ void QgsGrassRasterProvider::clearLastError()
   mLastError.clear();
 }
 
-void QgsGrassRasterProvider::appendIfError( QString error )
+void QgsGrassRasterProvider::appendIfError( const QString &error )
 {
   if ( !error.isEmpty() )
   {
@@ -608,12 +563,12 @@ QString QgsGrassRasterProvider::lastError()
 
 QString  QgsGrassRasterProvider::name() const
 {
-  return QString( "grassraster" );
+  return QStringLiteral( "grassraster" );
 }
 
 QString  QgsGrassRasterProvider::description() const
 {
-  return QString( "GRASS %1 raster provider" ).arg( GRASS_VERSION_MAJOR );
+  return QStringLiteral( "GRASS %1 raster provider" ).arg( GRASS_VERSION_MAJOR );
 }
 
 QDateTime QgsGrassRasterProvider::dataTimestamp() const
@@ -621,8 +576,8 @@ QDateTime QgsGrassRasterProvider::dataTimestamp() const
   QDateTime time;
   QString mapset = mGisdbase + "/" + mLocation + "/" + mMapset;
   QStringList dirs;
-  dirs << "cell" << "colr";
-  Q_FOREACH ( const QString& dir, dirs )
+  dirs << QStringLiteral( "cell" ) << QStringLiteral( "colr" );
+  Q_FOREACH ( const QString &dir, dirs )
   {
     QString path = mapset + "/" + dir + "/" + mMapName;
     QFileInfo fi( path );
@@ -649,17 +604,12 @@ void QgsGrassRasterProvider::thaw()
 
 //-------------------------------- QgsGrassRasterValue ----------------------------------------
 
-QgsGrassRasterValue::QgsGrassRasterValue()
-    : mProcess( 0 )
-{
-}
-
 QgsGrassRasterValue::~QgsGrassRasterValue()
 {
   stop();
 }
 
-void QgsGrassRasterValue::set( const QString & gisdbase, const QString & location, const QString & mapset, const QString & map )
+void QgsGrassRasterValue::set( const QString &gisdbase, const QString &location, const QString &mapset, const QString &map )
 {
   mGisdbase = gisdbase;
   mLocation = location;
@@ -677,7 +627,7 @@ void QgsGrassRasterValue::start()
   QString module = QgsGrass::qgisGrassModulePath() + "/qgis.g.info";
   QStringList arguments;
 
-  arguments.append( "info=query" );
+  arguments.append( QStringLiteral( "info=query" ) );
   arguments.append( "rast=" +  mMapName + "@" + mMapset );
   try
   {
@@ -700,7 +650,7 @@ void QgsGrassRasterValue::stop()
     mProcess->waitForFinished();
     QgsDebugMsg( "process finished" );
     delete mProcess;
-    mProcess = 0;
+    mProcess = nullptr;
   }
 }
 
@@ -719,20 +669,20 @@ double QgsGrassRasterValue::value( double x, double y, bool *ok )
     return value;
   }
 
-  QString coor = QString( "%1 %2\n" ).arg( QgsRasterBlock::printValue( x ),
+  QString coor = QStringLiteral( "%1 %2\n" ).arg( QgsRasterBlock::printValue( x ),
                  QgsRasterBlock::printValue( y ) );
   QgsDebugMsg( "coor : " + coor );
-  mProcess->write( coor.toAscii() ); // how to flush, necessary?
+  mProcess->write( coor.toLatin1() ); // how to flush, necessary?
   mProcess->waitForReadyRead();
   QString str = mProcess->readLine().trimmed();
   QgsDebugMsg( "read from stdout : " + str );
 
   // TODO: use doubles instead of strings
 
-  QStringList list = str.trimmed().split( ":" );
+  QStringList list = str.trimmed().split( ':' );
   if ( list.size() == 2 )
   {
-    if ( list[1] == "error" ) return value;
+    if ( list[1] == QLatin1String( "error" ) ) return value;
     value = list[1].toDouble( ok );
   }
   return value;
