@@ -21,23 +21,37 @@
 #include "qgslogger.h"
 #include "qgsmessagelog.h"
 #include "qgsgeometry.h"
+#include "qgssettings.h"
+#include "qgsexception.h"
 
 #include <QObject>
-#include <QSettings>
 
-QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* source, bool ownSource, const QgsFeatureRequest &request )
-    : QgsAbstractFeatureIteratorFromSource<QgsOracleFeatureSource>( source, ownSource, request )
-    , mRewind( false )
-    , mExpressionCompiled( false )
-    , mFetchGeometry( false )
+QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource *source, bool ownSource, const QgsFeatureRequest &request )
+  : QgsAbstractFeatureIteratorFromSource<QgsOracleFeatureSource>( source, ownSource, request )
 {
-  mConnection = QgsOracleConnPool::instance()->acquireConnection( QgsOracleConn::toPoolName( mSource->mUri ) );
+  mConnection = QgsOracleConnPool::instance()->acquireConnection( QgsOracleConn::toPoolName( mSource->mUri ), request.timeout(), request.requestMayBeNested() );
   if ( !mConnection )
   {
     close();
     return;
   }
 
+  if ( mRequest.destinationCrs().isValid() && mRequest.destinationCrs() != mSource->mCrs )
+  {
+    mTransform = QgsCoordinateTransform( mSource->mCrs, mRequest.destinationCrs(), mRequest.transformContext() );
+  }
+  try
+  {
+    mFilterRect = filterRectToSourceCrs( mTransform );
+  }
+  catch ( QgsCsException & )
+  {
+    // can't reproject mFilterRect
+    mClosed = true;
+    return;
+  }
+
+  QVariantList args;
   mQry = QSqlQuery( *mConnection );
 
   if ( mRequest.flags() & QgsFeatureRequest::SubsetOfAttributes )
@@ -49,9 +63,9 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
     // ensure that all attributes required for expression filter are being fetched
     if ( mRequest.filterType() == QgsFeatureRequest::FilterExpression )
     {
-      Q_FOREACH ( const QString& field, mRequest.filterExpression()->referencedColumns() )
+      Q_FOREACH ( const QString &field, mRequest.filterExpression()->referencedColumns() )
       {
-        int attrIdx = mSource->mFields.fieldNameIndex( field );
+        int attrIdx = mSource->mFields.lookupField( field );
         if ( !mAttributeList.contains( attrIdx ) )
           mAttributeList << attrIdx;
       }
@@ -59,9 +73,9 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
 
     // ensure that all attributes required for order by are fetched
     const QSet< QString > orderByAttributes = mRequest.orderBy().usedAttributes();
-    Q_FOREACH ( const QString& attr, orderByAttributes )
+    for ( const QString &attr : orderByAttributes )
     {
-      int attrIndex = mSource->mFields.fieldNameIndex( attr );
+      int attrIndex = mSource->mFields.lookupField( attr );
       if ( !mAttributeList.contains( attrIndex ) )
         mAttributeList << attrIndex;
     }
@@ -81,32 +95,30 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
       mFetchGeometry = true;
     }
 
-    if ( !mRequest.filterRect().isNull() )
+    if ( !mFilterRect.isNull() )
     {
       // sdo_filter requires spatial index
       if ( mSource->mHasSpatialIndex )
       {
-        QgsRectangle rect( mRequest.filterRect() );
-        QString bbox = QString( "mdsys.sdo_geometry(2003,%1,NULL,"
-                                "mdsys.sdo_elem_info_array(1,1003,3),"
-                                "mdsys.sdo_ordinate_array(%2,%3,%4,%5)"
-                                ")" )
-                       .arg( mSource->mSrid < 1 ? "NULL" : QString::number( mSource->mSrid ) )
-                       .arg( qgsDoubleToString( rect.xMinimum() ) )
-                       .arg( qgsDoubleToString( rect.yMinimum() ) )
-                       .arg( qgsDoubleToString( rect.xMaximum() ) )
-                       .arg( qgsDoubleToString( rect.yMaximum() ) );
+        QString bbox = QStringLiteral( "mdsys.sdo_geometry(2003,?,NULL,"
+                                       "mdsys.sdo_elem_info_array(1,1003,3),"
+                                       "mdsys.sdo_ordinate_array(?,?,?,?)"
+                                       ")" );
 
-        whereClause = QString( "sdo_filter(%1,%2)='TRUE'" ).arg( QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn ) ).arg( bbox );
+        whereClause = QStringLiteral( "sdo_filter(%1,%2)='TRUE'" )
+                      .arg( QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn ), bbox );
 
-        if (( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) != 0 )
+        args << ( mSource->mSrid < 1 ? QVariant( QVariant::Int ) : mSource->mSrid ) << mFilterRect.xMinimum() << mFilterRect.yMinimum() << mFilterRect.xMaximum() << mFilterRect.yMaximum();
+
+        if ( ( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) != 0 )
         {
           // sdo_relate requires Spatial
           if ( mConnection->hasSpatial() )
           {
-            whereClause += QString( " AND sdo_relate(%1,%2,'mask=ANYINTERACT')='TRUE'" )
-                           .arg( QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn ) )
-                           .arg( bbox );
+            whereClause += QStringLiteral( " AND sdo_relate(%1,%2,'mask=ANYINTERACT')='TRUE'" )
+                           .arg( QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn ),
+                                 bbox );
+            args << ( mSource->mSrid < 1 ? QVariant( QVariant::Int ) : mSource->mSrid ) << mFilterRect.xMinimum() << mFilterRect.yMinimum() << mFilterRect.xMaximum() << mFilterRect.yMaximum();
           }
           else
           {
@@ -122,23 +134,23 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
       }
     }
   }
-  else if ( !mRequest.filterRect().isNull() )
+  else if ( !mFilterRect.isNull() )
   {
-    QgsDebugMsg( "filterRect without geometry ignored" );
+    QgsDebugMsg( QStringLiteral( "filterRect without geometry ignored" ) );
   }
 
   switch ( mRequest.filterType() )
   {
     case QgsFeatureRequest::FilterFid:
     {
-      QString fidWhereClause = QgsOracleUtils::whereClause( mRequest.filterFid(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared );
+      QString fidWhereClause = QgsOracleUtils::whereClause( mRequest.filterFid(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared, args );
       whereClause = QgsOracleUtils::andWhereClauses( whereClause, fidWhereClause );
     }
     break;
 
     case QgsFeatureRequest::FilterFids:
     {
-      QString fidsWhereClause = QgsOracleUtils::whereClause( mRequest.filterFids(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared );
+      QString fidsWhereClause = QgsOracleUtils::whereClause( mRequest.filterFids(), mSource->mFields, mSource->mPrimaryKeyType, mSource->mPrimaryKeyAttrs, mSource->mShared, args );
       whereClause = QgsOracleUtils::andWhereClauses( whereClause, fidsWhereClause );
     }
     break;
@@ -150,30 +162,27 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
       //handled below
       break;
 
-    case QgsFeatureRequest::FilterRect:
-      // Handled in the if-statement above
-      break;
   }
 
-  if ( mSource->mRequestedGeomType != QGis::WKBUnknown && mSource->mRequestedGeomType != mSource->mDetectedGeomType )
+  if ( mSource->mRequestedGeomType != QgsWkbTypes::Unknown && mSource->mRequestedGeomType != mSource->mDetectedGeomType )
   {
     if ( !whereClause.isEmpty() )
-      whereClause += " AND ";
+      whereClause += QStringLiteral( " AND " );
 
     whereClause += '(';
 
-    whereClause += QgsOracleConn::databaseTypeFilter( "FEATUREREQUEST", mSource->mGeometryColumn, mSource->mRequestedGeomType );
+    whereClause += QgsOracleConn::databaseTypeFilter( QStringLiteral( "FEATUREREQUEST" ), mSource->mGeometryColumn, mSource->mRequestedGeomType );
 
-    if ( mRequest.filterRect().isNull() )
-      whereClause += QString( " OR %1 IS NULL" ).arg( mSource->mGeometryColumn );
+    if ( mFilterRect.isNull() )
+      whereClause += QStringLiteral( " OR %1 IS NULL" ).arg( mSource->mGeometryColumn );
     whereClause += ')';
   }
 
   if ( !mSource->mSqlWhereClause.isEmpty() )
   {
     if ( !whereClause.isEmpty() )
-      whereClause += " AND ";
-    whereClause += "(" + mSource->mSqlWhereClause + ")";
+      whereClause += QStringLiteral( " AND " );
+    whereClause += '(' + mSource->mSqlWhereClause + ')';
   }
 
   //NOTE - must be last added!
@@ -183,7 +192,7 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
   bool useFallback = false;
   if ( request.filterType() == QgsFeatureRequest::FilterExpression )
   {
-    if ( QSettings().value( "/qgis/compileExpressions", true ).toBool() )
+    if ( QgsSettings().value( QStringLiteral( "qgis/compileExpressions" ), true ).toBool() )
     {
       QgsOracleExpressionCompiler compiler( mSource );
       QgsSqlExpressionCompiler::Result result = compiler.compile( mRequest.filterExpression() );
@@ -217,16 +226,17 @@ QgsOracleFeatureIterator::QgsOracleFeatureIterator( QgsOracleFeatureSource* sour
   if ( mRequest.limit() >= 0 && limitAtProvider )
   {
     if ( !whereClause.isEmpty() )
-      whereClause += " AND ";
+      whereClause += QStringLiteral( " AND " );
 
-    whereClause += QString( "rownum<=%1" ).arg( mRequest.limit() );
-    fallbackStatement += QString( "rownum<=%1" ).arg( mRequest.limit() );
+    whereClause += QStringLiteral( "rownum<=?" );
+    fallbackStatement += QStringLiteral( "rownum<=?" );
+    args << QVariant::fromValue( mRequest.limit() );
   }
 
-  bool result = openQuery( whereClause, !useFallback );
+  bool result = openQuery( whereClause, args, !useFallback );
   if ( !result && useFallback )
   {
-    result = openQuery( fallbackStatement );
+    result = openQuery( fallbackStatement, args );
     if ( result )
     {
       mExpressionCompiled = false;
@@ -240,7 +250,7 @@ QgsOracleFeatureIterator::~QgsOracleFeatureIterator()
   close();
 }
 
-bool QgsOracleFeatureIterator::nextFeatureFilterExpression( QgsFeature& f )
+bool QgsOracleFeatureIterator::nextFeatureFilterExpression( QgsFeature &f )
 {
   if ( !mExpressionCompiled )
     return QgsAbstractFeatureIterator::nextFeatureFilterExpression( f );
@@ -248,7 +258,7 @@ bool QgsOracleFeatureIterator::nextFeatureFilterExpression( QgsFeature& f )
     return fetchFeature( f );
 }
 
-bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
+bool QgsOracleFeatureIterator::fetchFeature( QgsFeature &feature )
 {
   feature.setValid( false );
 
@@ -258,17 +268,17 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
   for ( ;; )
   {
     feature.initAttributes( mSource->mFields.count() );
-    feature.setGeometry( 0 );
+    feature.clearGeometry();
     feature.setValid( false );
 
     if ( mRewind )
     {
       mRewind = false;
-      if ( !QgsOracleProvider::exec( mQry, mSql ) )
+      if ( !QgsOracleProvider::exec( mQry, mSql, mArgs ) )
       {
-        QgsMessageLog::logMessage( QObject::tr( "Fetching features failed.\nSQL:%1\nError: %2" )
-                                   .arg( mQry.lastQuery() )
-                                   .arg( mQry.lastError().text() ),
+        QgsMessageLog::logMessage( QObject::tr( "Fetching features failed.\nSQL: %1\nError: %2" )
+                                   .arg( mQry.lastQuery(),
+                                         mQry.lastError().text() ),
                                    QObject::tr( "Oracle" ) );
         return false;
       }
@@ -282,50 +292,47 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
 
     if ( mFetchGeometry )
     {
-      QByteArray *ba = static_cast<QByteArray*>( mQry.value( col++ ).data() );
-      if ( ba->size() > 0 )
+      QByteArray ba( mQry.value( col++ ).toByteArray() );
+      if ( ba.size() > 0 )
       {
-        unsigned char *copy = new unsigned char[ba->size()];
-        memcpy( copy, ba->constData(), ba->size() );
-
-        QgsGeometry *g = new QgsGeometry();
-        g->fromWkb( copy, ba->size() );
+        QgsGeometry g;
+        g.fromWkb( ba );
         feature.setGeometry( g );
       }
       else
       {
-        feature.setGeometry( 0 );
+        feature.clearGeometry();
       }
 
-      if ( !mRequest.filterRect().isNull() )
+      if ( !mFilterRect.isNull() )
       {
-        if ( !feature.geometry() )
+        if ( !feature.hasGeometry() )
         {
-          QgsDebugMsg( "no geometry to intersect" );
+          QgsDebugMsgLevel( QStringLiteral( "no geometry to intersect" ), 4 );
           continue;
         }
 
-        if (( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) == 0 )
+        if ( ( mRequest.flags() & QgsFeatureRequest::ExactIntersect ) == 0 )
         {
           // even if we could use sdo_filter earlier, we still need to double-check the results
           // as sdo_filter can return results outside the filter (it's only a first-pass
           // filtering operation!)
 
           // only want features which intersect with bbox
-          if ( !feature.geometry()->boundingBox().intersects( mRequest.filterRect() ) )
+          if ( !feature.geometry().boundingBox().intersects( mFilterRect ) )
           {
             // skip feature that don't intersect with our rectangle
-            QgsDebugMsg( "no bbox intersect" );
+            QgsDebugMsgLevel( QStringLiteral( "no bbox intersect" ), 4 );
             continue;
           }
         }
         else if ( !mConnection->hasSpatial() || !mSource->mHasSpatialIndex )
         {
           // couldn't use sdo_relate earlier
-          if ( !feature.geometry()->intersects( mRequest.filterRect() ) )
+          if ( !feature.geometry().intersects( mFilterRect ) )
           {
             // skip feature that don't intersect with our rectangle
-            QgsDebugMsg( "no exact intersect" );
+            QgsDebugMsgLevel( QStringLiteral( "no exact intersect" ), 4 );
             continue;
           }
         }
@@ -336,23 +343,22 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
 
     switch ( mSource->mPrimaryKeyType )
     {
-      case pktInt:
+      case PktInt:
         // get 64bit integer from result
         fid = mQry.value( col++ ).toLongLong();
-        if ( mAttributeList.contains( mSource->mPrimaryKeyAttrs[0] ) )
-          feature.setAttribute( mSource->mPrimaryKeyAttrs[0], fid );
+        if ( mAttributeList.contains( mSource->mPrimaryKeyAttrs.value( 0 ) ) )
+          feature.setAttribute( mSource->mPrimaryKeyAttrs.value( 0 ), fid );
         break;
 
-      case pktRowId:
-      case pktFidMap:
+      case PktRowId:
+      case PktFidMap:
       {
-        QList<QVariant> primaryKeyVals;
-
-        if ( mSource->mPrimaryKeyType == pktFidMap )
+        QVariantList primaryKeyVals;
+        if ( mSource->mPrimaryKeyType == PktFidMap )
         {
           Q_FOREACH ( int idx, mSource->mPrimaryKeyAttrs )
           {
-            const QgsField &fld = mSource->mFields[idx];
+            QgsField fld = mSource->mFields.at( idx );
 
             QVariant v = mQry.value( col );
             if ( v.type() != fld.type() )
@@ -370,17 +376,17 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
           primaryKeyVals << mQry.value( col++ );
         }
 
-        fid = mSource->mShared->lookupFid( QVariant( primaryKeyVals ) );
+        fid = mSource->mShared->lookupFid( primaryKeyVals );
       }
       break;
 
-      case pktUnknown:
+      case PktUnknown:
         Q_ASSERT( !"FAILURE: cannot get feature with unknown primary key" );
         return false;
     }
 
-    feature.setFeatureId( fid );
-    QgsDebugMsgLevel( QString( "fid=%1" ).arg( fid ), 5 );
+    feature.setId( fid );
+    QgsDebugMsgLevel( QStringLiteral( "fid=%1" ).arg( fid ), 5 );
 
     // iterate attributes
     Q_FOREACH ( int idx, mAttributeList )
@@ -388,21 +394,17 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
       if ( mSource->mPrimaryKeyAttrs.contains( idx ) )
         continue;
 
-      const QgsField &fld = mSource->mFields[idx];
+      QgsField fld = mSource->mFields.at( idx );
 
       QVariant v = mQry.value( col );
-      if ( fld.type() == QVariant::ByteArray && fld.typeName().endsWith( ".SDO_GEOMETRY" ) )
+      if ( fld.type() == QVariant::ByteArray && fld.typeName().endsWith( QStringLiteral( ".SDO_GEOMETRY" ) ) )
       {
-        QByteArray *ba = static_cast<QByteArray*>( v.data() );
-        if ( ba->size() > 0 )
+        QByteArray ba( v.toByteArray() );
+        if ( ba.size() > 0 )
         {
-          unsigned char *copy = new unsigned char[ba->size()];
-          memcpy( copy, ba->constData(), ba->size() );
-
-          QgsGeometry *g = new QgsGeometry();
-          g->fromWkb( copy, ba->size() );
-          v = g->exportToWkt();
-          delete g;
+          QgsGeometry g;
+          g.fromWkb( ba );
+          v = g.asWkt();
         }
         else
         {
@@ -419,6 +421,8 @@ bool QgsOracleFeatureIterator::fetchFeature( QgsFeature& feature )
     feature.setValid( true );
     feature.setFields( mSource->mFields ); // allow name-based attribute lookups
 
+    geometryToDestinationCrs( feature, mTransform );
+
     return true;
   }
 }
@@ -440,49 +444,49 @@ bool QgsOracleFeatureIterator::close()
 
   if ( mConnection )
     QgsOracleConnPool::instance()->releaseConnection( mConnection );
-  mConnection = 0;
+  mConnection = nullptr;
 
   iteratorClosed();
 
   return true;
 }
 
-bool QgsOracleFeatureIterator::openQuery( QString whereClause, bool showLog )
+bool QgsOracleFeatureIterator::openQuery( const QString &whereClause, const QVariantList &args, bool showLog )
 {
   try
   {
-    QString query = "SELECT ", delim = "";
+    QString query = QStringLiteral( "SELECT " );
+    QString delim;
 
     if ( mFetchGeometry )
     {
       query += QgsOracleProvider::quotedIdentifier( mSource->mGeometryColumn );
-      delim = ",";
+      delim = ',';
     }
 
     switch ( mSource->mPrimaryKeyType )
     {
-      case pktRowId:
-        query += delim + QgsOracleProvider::quotedIdentifier( "ROWID" );
-        delim = ",";
+      case PktRowId:
+        query += delim + QgsOracleProvider::quotedIdentifier( QStringLiteral( "ROWID" ) );
+        delim = ',';
         break;
 
-      case pktInt:
-        query += delim + QgsOracleProvider::quotedIdentifier( mSource->mFields[ mSource->mPrimaryKeyAttrs[0] ].name() );
-        delim = ",";
+      case PktInt:
+        query += delim + QgsOracleProvider::quotedIdentifier( mSource->mFields.at( mSource->mPrimaryKeyAttrs[0] ).name() );
+        delim = ',';
         break;
 
-      case pktFidMap:
+      case PktFidMap:
         Q_FOREACH ( int idx, mSource->mPrimaryKeyAttrs )
         {
-          query += delim + mConnection->fieldExpression( mSource->mFields[idx] );
-          delim = ",";
+          query += delim + mConnection->fieldExpression( mSource->mFields.at( idx ) );
+          delim = ',';
         }
         break;
 
-      case pktUnknown:
-        QgsDebugMsg( "Cannot query without primary key." );
+      case PktUnknown:
+        QgsDebugMsg( QStringLiteral( "Cannot query without primary key." ) );
         return false;
-        break;
     }
 
     Q_FOREACH ( int idx, mAttributeList )
@@ -490,23 +494,24 @@ bool QgsOracleFeatureIterator::openQuery( QString whereClause, bool showLog )
       if ( mSource->mPrimaryKeyAttrs.contains( idx ) )
         continue;
 
-      query += delim + mConnection->fieldExpression( mSource->mFields[idx] );
+      query += delim + mConnection->fieldExpression( mSource->mFields.at( idx ) );
     }
 
-    query += QString( " FROM %1 \"FEATUREREQUEST\"" ).arg( mSource->mQuery );
+    query += QStringLiteral( " FROM %1 \"FEATUREREQUEST\"" ).arg( mSource->mQuery );
 
     if ( !whereClause.isEmpty() )
-      query += QString( " WHERE %1" ).arg( whereClause );
+      query += QStringLiteral( " WHERE %1" ).arg( whereClause );
 
-    QgsDebugMsg( QString( "Fetch features: %1" ).arg( query ) );
+    QgsDebugMsg( QStringLiteral( "Fetch features: %1" ).arg( query ) );
     mSql = query;
-    if ( !QgsOracleProvider::exec( mQry, query ) )
+    mArgs = args;
+    if ( !QgsOracleProvider::exec( mQry, query, args ) )
     {
       if ( showLog )
       {
-        QgsMessageLog::logMessage( QObject::tr( "Fetching features failed.\nSQL:%1\nError: %2" )
-                                   .arg( mQry.lastQuery() )
-                                   .arg( mQry.lastError().text() ),
+        QgsMessageLog::logMessage( QObject::tr( "Fetching features failed.\nSQL: %1\nError: %2" )
+                                   .arg( mQry.lastQuery(),
+                                         mQry.lastError().text() ),
                                    QObject::tr( "Oracle" ) );
       }
       return false;
@@ -522,23 +527,24 @@ bool QgsOracleFeatureIterator::openQuery( QString whereClause, bool showLog )
 
 // -----------
 
-QgsOracleFeatureSource::QgsOracleFeatureSource( const QgsOracleProvider* p )
-    : mUri( p->mUri )
-    , mFields( p->mAttributeFields )
-    , mGeometryColumn( p->mGeometryColumn )
-    , mSrid( p->mSrid )
-    , mHasSpatialIndex( p->mHasSpatialIndex )
-    , mDetectedGeomType( p->mDetectedGeomType )
-    , mRequestedGeomType( p->mRequestedGeomType )
-    , mSqlWhereClause( p->mSqlWhereClause )
-    , mPrimaryKeyType( p->mPrimaryKeyType )
-    , mPrimaryKeyAttrs( p->mPrimaryKeyAttrs )
-    , mQuery( p->mQuery )
-    , mShared( p->mShared )
+QgsOracleFeatureSource::QgsOracleFeatureSource( const QgsOracleProvider *p )
+  : mUri( p->mUri )
+  , mFields( p->mAttributeFields )
+  , mGeometryColumn( p->mGeometryColumn )
+  , mSrid( p->mSrid )
+  , mHasSpatialIndex( p->mHasSpatialIndex )
+  , mDetectedGeomType( p->mDetectedGeomType )
+  , mRequestedGeomType( p->mRequestedGeomType )
+  , mSqlWhereClause( p->mSqlWhereClause )
+  , mPrimaryKeyType( p->mPrimaryKeyType )
+  , mPrimaryKeyAttrs( p->mPrimaryKeyAttrs )
+  , mQuery( p->mQuery )
+  , mCrs( p->crs() )
+  , mShared( p->mShared )
 {
 }
 
-QgsFeatureIterator QgsOracleFeatureSource::getFeatures( const QgsFeatureRequest& request )
+QgsFeatureIterator QgsOracleFeatureSource::getFeatures( const QgsFeatureRequest &request )
 {
   return QgsFeatureIterator( new QgsOracleFeatureIterator( this, false, request ) );
 }
