@@ -26,7 +26,7 @@ __author__ = 'Alessandro Pasotti'
 __date__ = '25/05/2015'
 __copyright__ = 'Copyright 2015, The QGIS Project'
 # This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 import os
 
@@ -130,6 +130,9 @@ class QgsServerTestBase(unittest.TestCase):
         self.projectUseLayerIdsPath = os.path.join(d, "project_use_layerids.qgs")
         self.projectGroupsPath = os.path.join(d, "project_groups.qgs")
 
+        os.environ['QGIS_SERVER_SHOW_GROUP_SEPARATOR'] = '0'
+        os.environ['QGIS_SERVER_OVERRIDE_SYSTEM_LOCALE'] = 'en'
+
         # Clean env just to be sure
         env_vars = ['QUERY_STRING', 'QGIS_PROJECT_FILE']
         for ev in env_vars:
@@ -259,6 +262,12 @@ class QgsServerTestBase(unittest.TestCase):
             headers.append(("%s: %s" % (k, rh[k])).encode('utf-8'))
         return b"\n".join(headers) + b"\n\n", bytes(response.body())
 
+    def _assert_status_code(self, status_code, qs, requestMethod=QgsServerRequest.GetMethod, data=None, project=None):
+        request = QgsBufferServerRequest(qs, requestMethod, {}, data)
+        response = QgsBufferServerResponse()
+        self.server.handleRequest(request, response, project)
+        assert response.statusCode() == status_code, "%s != %s" % (response.statusCode(), status_code)
+
 
 class TestQgsServerTestBase(unittest.TestCase):
 
