@@ -14,7 +14,7 @@ __author__ = 'René-Luc Dhont'
 __date__ = '19/09/2017'
 __copyright__ = 'Copyright 2017, The QGIS Project'
 # This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 import os
 
@@ -92,6 +92,10 @@ class TestQgsServerWFS(QgsServerTestBase):
             query_string, request))
         return header, body
 
+    def test_operation_not_supported(self):
+        qs = '?MAP=%s&SERVICE=WFS&VERSION=1.1.0&REQUEST=NotAValidRequest' % urllib.parse.quote(self.projectPath)
+        self._assert_status_code(501, qs)
+
     def test_project_wfs(self):
         """Test some WFS request"""
         for request in ('GetCapabilities', 'DescribeFeatureType'):
@@ -452,6 +456,13 @@ class TestQgsServerWFS(QgsServerTestBase):
         self.wfs_request_compare("DescribeFeatureType", '1.1.0', "TYPENAME=does_not_exist&",
                                  'wfs_describeFeatureType_1_1_0_typename_wrong', project_file=project_file)
 
+    def test_describeFeatureTypeVirtualFields(self):
+        """Test DescribeFeatureType with virtual fields: bug GH-29767"""
+
+        project_file = "bug_gh29767_double_vfield.qgs"
+        self.wfs_request_compare("DescribeFeatureType", '1.1.0', "",
+                                 'wfs_describeFeatureType_1_1_0_virtual_fields', project_file=project_file)
+
     def test_getFeatureFeature_0_nulls(self):
         """Test that 0 and null in integer columns are reported correctly"""
 
