@@ -14,7 +14,7 @@ __author__ = 'Alessandro Pasotti'
 __date__ = '25/05/2015'
 __copyright__ = 'Copyright 2015, The QGIS Project'
 # This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 import os
 
@@ -434,6 +434,42 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
         self._img_diff_error(r, h, "WMS_GetLegendGraphic_ItemFont", max_size_diff=QSize(1, 1))
 
     def test_wms_GetLegendGraphic_BBox(self):
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": urllib.parse.quote(self.projectPath),
+            "SERVICE": "WMS",
+            "VERSION": "1.1.1",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "Country,Hello,db_point",
+            "LAYERTITLE": "FALSE",
+            "FORMAT": "image/png",
+            "SRCHEIGHT": "500",
+            "SRCWIDTH": "500",
+            "BBOX": "-151.7,-38.9,51.0,78.0",
+            "CRS": "EPSG:4326"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_BBox")
+
+    def test_wms_GetLegendGraphic_BBox2(self):
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": urllib.parse.quote(self.projectPath),
+            "SERVICE": "WMS",
+            "VERSION": "1.1.1",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "Country,Hello,db_point",
+            "LAYERTITLE": "FALSE",
+            "FORMAT": "image/png",
+            "SRCHEIGHT": "500",
+            "SRCWIDTH": "500",
+            "BBOX": "-76.08,-6.4,-19.38,38.04",
+            "SRS": "EPSG:4326"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_BBox2")
+
+    def test_wms_GetLegendGraphic_BBox_Fallback(self):
         qs = "?" + "&".join(["%s=%s" % i for i in list({
             "MAP": urllib.parse.quote(self.projectPath),
             "SERVICE": "WMS",
@@ -451,7 +487,7 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
         r, h = self._result(self._execute_request(qs))
         self._img_diff_error(r, h, "WMS_GetLegendGraphic_BBox")
 
-    def test_wms_GetLegendGraphic_BBox2(self):
+    def test_wms_GetLegendGraphic_BBox2_Fallback(self):
         qs = "?" + "&".join(["%s=%s" % i for i in list({
             "MAP": urllib.parse.quote(self.projectPath),
             "SERVICE": "WMS",
@@ -477,8 +513,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "QGIS%20Server%20Hello%20World",
             "FORMAT": "image/png",
-            "HEIGHT": "840",
-            "WIDTH": "1226",
+            "SRCHEIGHT": "840",
+            "SRCWIDTH": "1226",
             "BBOX": "10.38450,-49.6370,73.8183,42.9461",
             "SRS": "EPSG:4326",
             "SCALE": "15466642"
@@ -499,8 +535,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "QGIS%20Server%20-%20Grouped%20Layer",
             "FORMAT": "image/png",
-            "HEIGHT": "840",
-            "WIDTH": "1226",
+            "SRCHEIGHT": "840",
+            "SRCWIDTH": "1226",
             "BBOX": "609152,5808188,625492,5814318",
             "SRS": "EPSG:25832",
             "SCALE": "38976"
@@ -518,8 +554,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "All_grouped_layers",
             "FORMAT": "image/png",
-            "HEIGHT": "840",
-            "WIDTH": "1226",
+            "SRCHEIGHT": "840",
+            "SRCWIDTH": "1226",
             "BBOX": "609152,5808188,625492,5814318",
             "SRS": "EPSG:25832",
             "SCALE": "38976"
@@ -537,8 +573,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "testlayer",
             "FORMAT": "image/png",
-            "HEIGHT": "550",
-            "WIDTH": "850",
+            "SRCHEIGHT": "550",
+            "SRCWIDTH": "850",
             "BBOX": "-608.4,-1002.6,698.2,1019.0",
             "CRS": "EPSG:4326"
         }.items())])
@@ -553,8 +589,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "testlayer",
             "FORMAT": "image/png",
-            "HEIGHT": "550",
-            "WIDTH": "850",
+            "SRCHEIGHT": "550",
+            "SRCWIDTH": "850",
             "BBOX": "-1261.7,-2013.5,1351.5,2029.9",
             "CRS": "EPSG:4326"
         }.items())])
@@ -570,8 +606,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "testlayer",
             "FORMAT": "image/png",
-            "HEIGHT": "550",
-            "WIDTH": "850",
+            "SRCHEIGHT": "550",
+            "SRCWIDTH": "850",
             "BBOX": "31.8,-12.0,58.0,28.4",
             "CRS": "EPSG:4326"
         }.items())])
@@ -587,8 +623,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "testlayer",
             "FORMAT": "image/png",
-            "HEIGHT": "550",
-            "WIDTH": "850",
+            "SRCHEIGHT": "550",
+            "SRCWIDTH": "850",
             "BBOX": "25.3,-22.1,64.5,38.5",
             "CRS": "EPSG:4326"
         }.items())])
@@ -604,8 +640,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "testlayer",
             "FORMAT": "image/png",
-            "HEIGHT": "550",
-            "WIDTH": "850",
+            "SRCHEIGHT": "550",
+            "SRCWIDTH": "850",
             "BBOX": "44.8,8.0,45.0,8.4",
             "CRS": "EPSG:4326"
         }.items())])
@@ -620,8 +656,8 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
             "REQUEST": "GetLegendGraphic",
             "LAYER": "testlayer",
             "FORMAT": "image/png",
-            "HEIGHT": "550",
-            "WIDTH": "850",
+            "SRCHEIGHT": "550",
+            "SRCWIDTH": "850",
             "BBOX": "43.6,6.2,46.2,10.2",
             "CRS": "EPSG:4326"
         }.items())])
@@ -629,6 +665,121 @@ class TestQgsServerWMSGetLegendGraphic(QgsServerTestBase):
         r, h = self._result(self._execute_request(qs))
         self._img_diff_error(r, h, "WMS_GetLegendGraphic_ScaleSymbol_Max", max_size_diff=QSize(15, 15))
 
+    def test_wms_GetLegendGraphic_ScaleSymbol_DefaultMapUnitsPerMillimeter(self):
+        # map units per mm on 1:20000000 with SRCHEIGHT=598&SRCWIDTH=1640&BBOX=16.5,-69.7,73.3,86.1 would be around what is set as default: 0.359 map units per mm
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": self.testdata_path + 'test_project_scaledsymbols.qgs',
+            "SERVICE": "WMS",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "testlayer",
+            "FORMAT": "image/png",
+            "CRS": "EPSG:4326"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_ScaleSymbol_DefaultMapUnitsPerMillimeter", max_size_diff=QSize(15, 15))
+
+    def test_wms_GetLegendGraphic_ScaleSymbol_Scaled_2056(self):
+        # 1:1000 scale on an EPSG:2056 calculating DPI that is around 96
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": self.testdata_path + 'test_project_scaledsymbols_2056.qgs',
+            "SERVICE": "WMS",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "testlayer_2056",
+            "FORMAT": "image/png",
+            "SRCHEIGHT": "600",
+            "SRCWIDTH": "1500",
+            "BBOX": "2662610.7,1268841.8,2663010.5,1269000.05",
+            "CRS": "EPSG:2056"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_ScaleSymbol_Scaled_2056", max_size_diff=QSize(15, 15))
+
+    def test_wms_GetLegendGraphic_ScaleSymbol_DefaultScale_2056(self):
+        # 1:1000 as default value - it's not exactly the same result than passing the bbox and size because of exact DPI 96 (default)
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": self.testdata_path + 'test_project_scaledsymbols_2056.qgs',
+            "SERVICE": "WMS",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "testlayer_2056",
+            "FORMAT": "image/png",
+            "CRS": "EPSG:2056"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_ScaleSymbol_DefaultScale_2056", max_size_diff=QSize(15, 15))
+        
+    def test_wms_GetLegendGraphic_LAYERFONTCOLOR(self):
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": urllib.parse.quote(self.projectPath),
+            "SERVICE": "WMS",
+            "VERSION": "1.1.1",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "Country,Hello",
+            "FORMAT": "image/png",
+            "HEIGHT": "500",
+            "WIDTH": "500",
+            "CRS": "EPSG:3857",
+            "LAYERFONTCOLOR": "red"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_LAYERFONTCOLOR", max_size_diff=QSize(10, 2))
+
+    def test_wms_GetLegendGraphic_ITEMFONTCOLOR(self):
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": urllib.parse.quote(self.projectPath),
+            "SERVICE": "WMS",
+            "VERSION": "1.1.1",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "Country,Hello",
+            "FORMAT": "image/png",
+            "HEIGHT": "500",
+            "WIDTH": "500",
+            "CRS": "EPSG:3857",
+            "ITEMFONTCOLOR": "red",
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_ITEMFONTCOLOR", max_size_diff=QSize(10, 2))
+
+    def test_wms_GetLegendGraphic_ITEMFONTCOLOR_and_LAYERFONTCOLOR(self):
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": urllib.parse.quote(self.projectPath),
+            "SERVICE": "WMS",
+            "VERSION": "1.1.1",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "Country,Hello",
+            "FORMAT": "image/png",
+            "HEIGHT": "500",
+            "WIDTH": "500",
+            "CRS": "EPSG:3857",
+            "ITEMFONTCOLOR": "red",
+            "LAYERFONTCOLOR": "blue"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_ITEMFONTCOLOR_and_LAYERFONTCOLOR", max_size_diff=QSize(10, 2))
+
+    def test_wms_GetLegendGraphic_ITEMFONTCOLOR_and_LAYERFONTCOLOR_hex(self):
+        qs = "?" + "&".join(["%s=%s" % i for i in list({
+            "MAP": urllib.parse.quote(self.projectPath),
+            "SERVICE": "WMS",
+            "VERSION": "1.1.1",
+            "REQUEST": "GetLegendGraphic",
+            "LAYER": "Country,Hello",
+            "FORMAT": "image/png",
+            "HEIGHT": "500",
+            "WIDTH": "500",
+            "CRS": "EPSG:3857",
+            "ITEMFONTCOLOR": r"%23FF0000",
+            "LAYERFONTCOLOR": r"%230000FF"
+        }.items())])
+
+        r, h = self._result(self._execute_request(qs))
+        self._img_diff_error(r, h, "WMS_GetLegendGraphic_ITEMFONTCOLOR_and_LAYERFONTCOLOR", max_size_diff=QSize(10, 2))
+
 
 if __name__ == '__main__':
     unittest.main()
