@@ -21,7 +21,7 @@ __author__ = 'Massimo Endrighi'
 __date__ = 'October 2012'
 __copyright__ = '(C) 2012, Massimo Endrighi'
 # This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '514ccce9bdbfeb2a72304f9e1b731e8229cd1fc2'
+__revision__ = '54585165bba895db4c83ffda980be7778246f2c5'
 
 import qgis  # NOQA
 
@@ -698,6 +698,11 @@ class TestQgsSymbolLayer(unittest.TestCase):
         mMessage = 'Expected "%s" got "%s"' % (mExpectedValue, mValue)
         assert mExpectedValue == mValue, mMessage
 
+        mExpectedValue = False
+        mValue = mSymbolLayer.pointOnAllParts()
+        mMessage = 'Expected "%s" got "%s"' % (mExpectedValue, mValue)
+        assert mExpectedValue == mValue, mMessage
+
         # test colors, need to make sure colors are passed/retrieved from subsymbol
         mSymbolLayer.setColor(QColor(150, 50, 100))
         self.assertEqual(mSymbolLayer.color(), QColor(150, 50, 100))
